function BookmarkRegistry() {

  // Set up a listener to re-register bookmarks when the url changes (hashchange or pushstate)
  var oldLocation = location.href;
  setInterval(function(obj) {
    return function () {
      if(location.href != oldLocation) {
        obj.registerUrlBookmarks();
        if (window.contentToolbar && window.contentToolbar.isOpen)
          window.contentToolbar.nullMode();
        oldLocation = location.href;
      }
    }
  }(this), 500);

  // Set up a listener to register a new page title if the page title changes (common with pushstate implementations)
  var oldTitle = document.title;
  setInterval(function(obj) {
    return function () {
      if(document.title != oldTitle) {
        if (obj.myBookmark == '__unsaved__')
          obj.bookmarks['__unsaved__'].title = obj.getPageTitle();
        oldTitle = document.title;
      }
    }
  }(this), 500);

}

BookmarkRegistry.prototype = {

  my_domain_bookmarks: {},
  other_domain_bookmarks: {},
  bookmarks: {},
  myBookmark: null,
  currentBookmark: null,
  currentHighlights: {},

  registerUrlBookmarks: function () {

    if (!document.URL)
      return;

    if (document.URL.match(/^https?:\/\/[^.]*\.?annotary\.com\/bookmark-proxy\/[0-9]+\/?.*$/i)) {
      var uri_split = document.URL.split('?');
      var uri = uri_split[0];
      var bookmark_id_split = uri.substring(uri.indexOf('/bookmark-proxy/') + 16).split('/');
      var bookmark_id = bookmark_id_split[0];
      var bookmark_slug = bookmark_id_split[1];
      uri = '/bookmarks/' + bookmark_id + '.json?slug=' + bookmark_slug;
      if (uri_split[1])
        var highlight = uri_split[1].substring(10);
    }
    else {
      var encoded_url = document.URL.replace(/ /g, '%20');
      encoded_url = encodeURIComponent(encoded_url);
      var uri = '/bookmarks.json?require_auth=1&summary=1&url=' + encoded_url; // require_auth is a hack to make the api throw a 401
    }

    // If there's a bookmark specified in the background, force include that bookmark then set bookmark_id and highlight
    // so we restore that bookmark after loading everything
    window.messenger.sendRequest({type: 'data', inst: 'linkRegistry', attr: 'registry'}, function (obj, bookmark_id, highlight) {
      return function (registry) {

        var key = document.URL.toLowerCase();
        if (!bookmark_id && registry[key]) {
          bookmark_id = registry[key].id;
          highlight = registry[key].highlight;
          uri += '&force_id=' + bookmark_id + '&force_slug=' + registry[key].slug;
          window.messenger.sendRequest({type: 'func', inst: 'linkRegistry', func: 'clear', args: [key]});
        }

        window.api.request({type: 'GET', uri: uri}, function (obj, bookmark_id, highlight) {
          return function (bookmarks) {

            // If we got an object (like from the bookmark-proxy request), put it in a list
            if (!$.isArray(bookmarks))
              bookmarks = [bookmarks];

            // Clear bookmark storage vars
            obj.my_domain_bookmarks = {};
            obj.other_domain_bookmarks = {};
            obj.bookmarks = {};
            obj.myBookmark = null;
            obj.currentBookmark = null;

            // Store data for related bookmarks on the same domain
            document_url = document.URL.replace(/^https?:\/\//,'')
            document_url = document.URL.replace(/ /g,'%20')
            for (i in bookmarks) {
              bookmark = bookmarks[i];
              bookmark_url = bookmark.url.replace(/^https?:\/\//,'')
              bookmark_url = bookmark.url.replace(/ /g,'%20')
              if (bookmark_url == document_url)
                continue;
              if (bookmark.my_bookmark)
                obj.my_domain_bookmarks[bookmark.id] = bookmark;
              else
                obj.other_domain_bookmarks[bookmark.id] = bookmark;
            }

            // Remove bookmarks of other urls so we can save just the current page bookmarks (keep the bookmark_id bookmark if there is one)
            for (i = 0; i < bookmarks.length; i++) {
              if (bookmark_id && bookmarks[i].id == bookmark_id)
                continue;
              bookmark_url = bookmarks[i].url.replace(/^https?:\/\//,'')
              bookmark_url = bookmarks[i].url.replace(/ /g,'%20')
              if (bookmark_url != document_url) {
                bookmarks.splice(i, 1);
                i--;
              }
            }

            // Store data for all bookmarks, including my bookmark
            for (i in bookmarks) {
              var bookmark = bookmarks[i];
              obj.bookmarks[bookmark.id] = bookmark;
            }
            obj.myBookmark = '__unsaved__';
            for (id in obj.bookmarks) {
              if (obj.bookmarks[id].my_bookmark == true)
                obj.myBookmark = id;
            }

            // Set defaults for my bookmark if it hasn't been saved yet
            if (obj.myBookmark == '__unsaved__') {
              obj.bookmarks['__unsaved__'] = {
                id: null,
                url: document.URL,
                title: obj.getPageTitle(),
                note: '',
                collection: '',
              }
            }

            // If we have a bookmark id already (iframe or registry), enable that bookmark, otherwise enable my bookmark
            if (bookmark_id) {
              obj.loadBookmark(bookmark_id, function (obj, highlight) {
                return function () {
                  obj.scrollToHighlight(highlight);
                }
              }(obj, highlight));
            }
            else
              obj.loadBookmark(obj.myBookmark);

            if (window.contentToolbar)
              window.contentToolbar.setBookmarkIndicators();

          }
        }(obj, bookmark_id, highlight));

      }
    }(this, bookmark_id, highlight));

  },

  getPageTitle: function () {

    og_title = $('meta[property="og:title"]').attr('content');
    if (og_title)
      return og_title;
    else if (document.title != '')
      return document.title;
    else
      return document.URL;

  },

  scrollToHighlight: function (highlight) {

    if (highlight) {
      var top = $(".annotary-highlight-id-" + highlight).offset().top - 50;
      if (top < 0)
        top = 0;
      $('html, body').animate({scrollTop: top}, 1000);
      window.highlighter.openNoteWindow(highlight);
    }

  },

  setMyBookmarkData: function (bookmark) {

    this.myBookmark = bookmark.id;
    this.bookmarks[bookmark.id] = {
      id: bookmark.id,
      url: bookmark.url,
      title: bookmark.title,
      slug: bookmark.slug,
      note: bookmark.note,
      collection: bookmark.collection
    }
    if (this.bookmarks['__unsaved__'])
      delete this.bookmarks['__unsaved__'];

  },

  iHaveBookmarked: function () {

    return !this.myBookmark || this.myBookmark == '__unsaved__' ? false : true;

  },

  addHighlightData: function (highlight, range) {

    this.currentHighlights[highlight.id] = {
      id: highlight.id,
      serialized: highlight.serialized,
      note: highlight.note,
      range: range
    };

  },

  loadBookmark: function (id, callback) {

    if (typeof callback != 'function')
      callback = function () {}

    if (!this.bookmarks[id] || this.currentBookmark == id) {
      callback();
      return;
    }

    this.clearBookmark();

    var bookmark = this.bookmarks[id];
    this.currentBookmark = id;

    // If it's showing someone else's bookmark, open a notice that it's someone else's highlights
    if (id != this.myBookmark && bookmark.highlights_count > 0) {

      var plural = bookmark.highlights_count == 1 ? '' : 's';
      $('body').append(
        $('<div id="annotary-highlight-notice"></div>')
          .append('Displaying ')
          .append(
            $('<b></b>').text(bookmark.highlights_count + ' highlight' + plural)
          )
          .append(' from the collection ')
          .append(
            $('<a></a>')
              .attr('href', 'https://annotary.com/collections/' + bookmark.collection.id + '/' + bookmark.collection.slug + '?utm_source=' + document.domain + '&utm_medium=extension')
              .text(bookmark.collection.title)
          )
          .append(' by ')
          .append(
            $('<a></a>')
              .attr('href', 'https://annotary.com/profiles/' + bookmark.user.username + '?utm_source=' + document.domain + '&utm_medium=extension')
              .text(bookmark.user.first_name + ' ' + bookmark.user.last_name)
          )
      );

      // Switch the toolbar to network mode if it's open already
      if (typeof ContentToolbar != 'undefined' && window.contentToolbar instanceof ContentToolbar && window.contentToolbar.isOpen && window.contentToolbar.currentMode != 'network')
        window.messenger.sendRequest({type: 'func', inst: 'backgroundToolbar', func: 'networkMode', args: ['__tab__']});

    }

    // Add highlights (populated into this.highlights by the window.highlighter class)
    if (id == '__unsaved__')
      callback();
    else {
      window.api.request({type: 'GET', uri: '/bookmarks/' + id + '/highlights.json?slug=' + bookmark.slug}, function (callback) {
        return function (highlights) {
          for (i in highlights) {
            var highlight = highlights[i];
            var savedRange = JSON.parse(highlight.serialized);
            var range = window.highlighter.restoreRange(savedRange);
            if (range)
              window.highlighter.highlightRange(range, highlight);
          }
          callback();
        }
      }(callback));
    }

  },

  showingSomeoneElsesBookmark: function () {

    if (this.currentBookmark && this.currentBookmark != this.myBookmark)
      return true;
    else
      return false;

  },

  countBookmarks: function () {

    var count = 0;
    for (i in this.bookmarks) {
      if (i != this.myBookmark)
        count++;
    }
    return count;

  },

  countDomainBookmarks: function (narrow) {

    var count = 0;
    if (narrow != 'other') {
      for (i in this.my_domain_bookmarks)
        count++;
    }
    if (narrow != 'my') {
      for (i in this.other_domain_bookmarks)
        count++;
    }
    return count;

  },

  clearBookmark: function () {

    this.currentBookmark = null;
    $('#annotary-highlight-notice').remove();
    for (i in this.currentHighlights)
      window.highlighter.clearHighlight(this.currentHighlights[i]);

  }

}
