BrowserApi = function () {

  if (typeof chrome != 'undefined' && typeof chrome.extension != 'undefined')
    this.browser = 'chrome';
  else if (typeof ff_self != 'undefined' || typeof self.port != 'undefined')
    this.browser = 'firefox';
  else if (typeof safari != 'undefined' && typeof safari.extension != 'undefined')
    this.browser = 'safari';
  else
    this.browser = null; // For when it's not running in an extension

}

BrowserApi.prototype = {

  browser: null,
  message_reply_callbacks: {}, // Used to register reply callbacks when sending messages that get replies
  default_message_callback: function () {}, // Used to store the callback for receiving default messages in safari

  getHost: function () {

    switch (this.browser) {

      case 'chrome':

        var extensionId = chrome.i18n.getMessage('@@extension_id');
        if (extensionId == 'fncckipieakpgpjifiadfiigdeanbond') // Chrome store version
          return 'https://annotary.com';
        else
          return 'https://localdev.annotary.com';
        break;

      case 'firefox':

        if (ff_is_dev)
          return 'https://localdev.annotary.com';
        else
          return 'https://annotary.com';
        break;

      case 'safari':

        // TODO: Dev vs production for safari extension
        return 'https://annotary.com';
        break;

      default:

        return 'https://' + document.domain;
        break;

    }

  },

  getURL: function (url) {

    switch (this.browser) {

      case 'chrome':
        return chrome.extension.getURL(url);
        break;

      case 'firefox':
        return "resource://jid1-d0wnmlonblgjig-at-jetpack/annotary/data" + url;
        break;

      case 'safari':
        return safari.extension.baseURI + url.substring(1);
        break;

      default:
        return SHARED_LIBS_URL + url;

    }

  },

  getCookie: function (cookie, callback) {

    switch (this.browser) {

      case 'chrome':
        chrome.cookies.get(cookie, callback);
        break;

      case 'firefox':
        var uri = ff_ioService.newURI(cookie.url, null, null);
        var name = cookie.name;
        var cookieStr = ff_cookieSvc.getCookieStringFromHttp(uri, null, null); // TODO: This function could be removed in the future, and may need to be replaced by using getCookieFromString with an instance of Ci.nsIHttpChannelInternal as the channel. See https://developer.mozilla.org/en/XPCOM_Interface_Reference/nsICookieService#section_8
        if (!cookieStr)
          cookieStr = '';
        var cookieAr = cookieStr.split(';');
        for (var cookie in cookieAr) {
          cookie = cookieAr[cookie];
          cookie = cookie.replace(/^\s\s*/, '').replace(/\s\s*$/, ''); //Trim
          cookie = cookie.split('=');
          if (cookie[0] == name) {
            if (cookie[1] == '')
              return callback(null);
            else
              return callback(cookie[1]);
          }
        }
        return callback(null);
        break;

      // TODO: A way to check and delete cookies in safari, or another workaround to block API calls when logged out?

      default:
        callback('Fake Cookie nom nom nom');
        break;

    }

  },

  removeCookie: function (cookie) {

    switch (this.browser) {

      case 'chrome':
        chrome.cookies.remove(cookie);
        break;

      case 'firefox':
        var uri = ff_ioService.newURI(cookie.url, null, null);
        var name = cookie.name;
        ff_cookieSvc.setCookieStringFromHttp(uri, null, null, name + '=', '01/01/12 00:00', null); // TODO: This function could be removed in the future, and may need to be replaced by using setCookieString with an instance of Ci.nsIHttpChannelInternal as the channel. See https://developer.mozilla.org/en/XPCOM_Interface_Reference/nsICookieService#section_13
        break;

      // TODO: A way to check and delete cookies in safari, or another workaround to block API calls when logged out?

    }

  },

  getAllTabs: function (callback) {

    switch (this.browser) {

      case 'chrome':
        chrome.windows.getAll({populate: true}, function (callback) {
          return function (windows) {
            var tabs = [];
            for (i in windows)
              tabs = tabs.concat(windows[i].tabs);
            callback(tabs);
          }
        }(callback));
        break;

      case 'firefox':
        var tabs = [];
        for (i in ff_tabs) {
          tabs.push(ff_tabs[i]);
        }
        callback(tabs);
        break;

      case 'safari':
        var tabs = [];
        for (i in safari.application.browserWindows) {
          tabs = tabs.concat(safari.application.browserWindows[i].tabs);
        }
        callback(tabs);
        break;

    }

  },

  createOrOpenLoginTab: function () {

    var url = this.getHost() + '/login?utm_source=direct&utm_medium=extension';

    switch (this.browser) {

      case 'chrome':
        var urlpattern = '*://*.annotary.com/login*';
        chrome.tabs.query({url: urlpattern}, function (url) {
          return function (tabs) {
            if (tabs.length == 0) {
              var top = screen.height/2 - 300;
              var left = screen.width/2 - 300;
              chrome.windows.create({
                url: url,
                top: top,
                left: left,
                width: 600,
                height: 600,
                focused: true,
                type: 'popup'
              }, function (win) {
                window.api.loginTabId = win.tabs[0].id;
              })
            }
            else {
              for (j in tabs)
                chrome.tabs.update(tabs[j].id, {selected: true}); // Select if it's already open
            }
          }
        }(url));
        break;

      case 'firefox':
        var urlpattern = /^https?:\/\/(|[^.\/]+\.)annotary\.com\/login/i;
        for (i in ff_tabs) {
          if (ff_tabs[i].url.match(urlpattern))
            return ff_tabs[i].activate();
        }
        ff_tabs.open({
          url: url,
          onOpen: function (obj) {
            return function (tab) {
              window.api.loginTabId = ff_next_tab_id;
            }
          }(this)
        });
        break;

      case 'safari':
        var urlpattern = /^https?:\/\/(|[^.\/]+\.)annotary\.com\/login/i;
        for (i in safari.application.browserWindows) {
          var browserWindow = safari.application.browserWindows[i];
          for (j in browserWindow.tabs) {
            var tab = browserWindow.tabs[j];
            if (tab.url && tab.url.match(urlpattern)) {
              browserWindow.activate();
              tab.activate();
              return;
            }
          }
        }
        window.api.loginTabId = safari_next_tab_id;
        newTab = safari.application.activeBrowserWindow.openTab();
        newTab.url = url;
        return newTab.activate();
        break;

    }

  },

  updateTab: function (tab, update) {

    switch(this.browser) {

      case 'chrome':
        chrome.tabs.update(tab.id, update);
        break;

      case 'firefox':
        if (typeof update.url != 'undefined')
          tab.url = update.url;
        break;

      case 'safari':
        if (typeof update.url != 'undefined')
          tab.url = update.url;
        break;

    }

  },

  closeTab: function (tabId) {

    switch(this.browser) {

      case 'chrome':
        chrome.tabs.remove(tabId);
        break;

      case 'firefox':
        ff_tab_registry[tabId].close();
        break;

      case 'safari':
        safari_tab_registry[tabId].close();
        break;

    }

  },

  isNewTab: function (tab) {

    switch (this.browser) {

      case 'chrome':
        return tab.url == 'chrome://newtab/' ? true : false;
        break;

      case 'firefox':
        return tab.url.match(/^about:.*$/) ? true : false;
        break;

      case 'safari':
        return (typeof tab.url == 'undefined') ? true : false;
        break;

    }

  },

  setBrowserButtonListener: function (callback) {

    switch (this.browser) {

      case 'chrome':
        chrome.browserAction.onClicked.addListener(callback);
        break;

      case 'firefox':
        // No button by default in Firefox, so create one in the global scope
        ff_btn = ff_toolbarbutton.ToolbarButton({
          id: 'annotary-toolbar-button',
          label: 'Annotary Toolbar',
          image: ff_self.data.url('icon.png'),
          onCommand: function (callback) {
            return function () {
              return callback(ff_tabs.activeTab);
            }
          }(callback)
        });
        ff_btn.moveTo({
          toolbarID: "nav-bar",
          forceMove: options.loadReason == 'install'
        });
        break;

      case 'safari':
        safari.application.addEventListener("command", function (e) {
          if (e.command == 'toolbarButton')
            callback(e.target.browserWindow.activeTab);
        }, false);
        break;

    }

  },

  setMessageListener: function (callback) {

    switch (this.browser) {

      case 'chrome':
        chrome.extension.onRequest.addListener(callback);
        break;

      case 'firefox':
        if (typeof ff_message_listener != 'undefined') // Background script just sets the callback as a variable, since it must create a new listener for each tab
          ff_message_listener = callback;
        else { // Content script
          self.port.on("default", function (callback) {
            return function (request) {
              callback(request, null, function (request) {
                return function (response) {
                  self.port.emit("reply-" + request.id, response);
                }
              }(request));
            }
          }(callback));
        }
        break;

      case 'safari':

        if (typeof safari.application != 'undefined') { // Background script
          this.default_message_callback = callback;
          safari.application.addEventListener("message", function (obj) {
            return function (e) {
              if (e.name == 'default') {
                var request = e.message;
                var sender = {tab: {id: obj.getTabId(e.target)}};
                obj.default_message_callback(request, sender, function (senderTab, request) {
                  return function (response) {
                    if (senderTab.page)
                      senderTab.page.dispatchMessage("reply-" + request.id, response);
                  }
                }(e.target, request));
              }
              else {
                var replyId = e.name.split('-')[1];
                var callback = obj.message_reply_callbacks[replyId];
                if (typeof callback == 'function')
                  callback(e.message);
              }
            }
          }(this), false);
        }
        else { // Content Script
          this.default_message_callback = callback;
          safari.self.addEventListener("message", function (obj) {
            return function (e) {
              if (e.name == 'default') {
                var request = e.message;
                obj.default_message_callback(request, null, function (request) {
                  return function (response) {
                    safari.self.tab.dispatchMessage("reply-" + request.id, response);
                  }
                }(request));
              }
              else {
                var replyId = e.name.split('-')[1];
                var callback = obj.message_reply_callbacks[replyId];
                if (typeof callback == 'function')
                  callback(e.message);
              }
            }
          }(this), false);
        }
        break;

    }

  },

  sendBackgroundRequest: function (request, callback) {

    switch (this.browser) {

      case 'chrome':
        chrome.extension.sendRequest(request, callback);
        break;

      case 'firefox':
        if (typeof callback == 'function')
          self.port.on("reply-" + request.id, callback);
        self.port.emit("default", request);
        break;

      case 'safari':
        if (typeof callback == 'function')
          this.message_reply_callbacks[request.id] = callback;
        safari.self.tab.dispatchMessage("default", request);
        break;

    }

  },

  sendTabRequest: function (tabId, request, callback) {

    switch (this.browser) {

      case 'chrome':
        chrome.tabs.sendRequest(tabId, request, callback);
        break;

      case 'firefox':
        var worker = ff_worker_registry[tabId];
        if (!worker)
          return;
        if (typeof callback == 'function')
          worker.port.on("reply-" + request.id, callback);
        worker.port.emit("default", request);
        break;

      case 'safari':
        var tab = safari_tab_registry[tabId];
        if (!tab)
          return;
        if (typeof callback == 'function')
          this.message_reply_callbacks[request.id] = callback;
        tab.page.dispatchMessage("default", request);
        break;

    }

  },

  setIcon: function (details) {

    switch (this.browser) {

      case 'chrome':
        chrome.browserAction.setIcon(details);
        break;

      case 'firefox':
        var url = ff_self.data.url(details.path.substring(1));
        ff_btn.setIcon({url: url});
        break;

      // Safari doesn't change the toolbar icon because it doesn't support color in the first place

    }

  },

  ajaxRequest: function (host, args, callback, errorCallback) {

    switch (this.browser) {

      case 'firefox':
        var xhr = new ff_xhr.XMLHttpRequest();
        xhr.open(args.type, host + '/api' + args.uri, true);
        xhr.onreadystatechange = function() {
          if (xhr.readyState != 4)
            return;
          if ((xhr.status + '').substring(0,1) != '2')
            return errorCallback(xhr);
          else if (xhr.status == 204)
            return callback(xhr.responseText);
          else
            return callback(JSON.parse(xhr.responseText));
        };
        if (args.data && args.data != {}) {
          var params = []
          for (k in args.data)
            params.push(encodeURIComponent(k) + '=' + encodeURIComponent(args.data[k]));
          xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
          xhr.send(params.join('&'));
        }
        else
          xhr.send();
        break;

      default:
        $.ajax({
          type: args.type,
          url: host + '/api' + args.uri,
          data: args.data,
          context: args.context,
          dataType: 'json',
          error: errorCallback,
          success: callback
        });
        break;

    }

  },

  getTabId: function (tab) {

    switch (this.browser) {

      case 'firefox':
        for (var tabId in ff_tab_registry) {
          if (tab === ff_tab_registry[tabId])
            return tabId;
        }
        return null;
        break;

      case 'safari':
        for (var tabId in safari_tab_registry) {
          if (tab === safari_tab_registry[tabId])
            return tabId;
        }
        return null;
        break;

      default:
        return tab.id;
        break;

    }

  }

}
