function CollectionsPicker (collectionId, parentEl) {

  this.collectionId = collectionId;

  // Create the collection picker html
  $(parentEl)
    .html(
      '<div class="annotary-collection"></div>' +
      '<div class="annotary-collection-expanded">' +
        '<div class="annotary-collection-create">' +
          '<input class="annotary-form-input" value="Filter or Create" />' +
        '</div>' +
        '<div class="annotary-collection-options"></div>' +
      '</div>'
    ).append(
      $('<input type="hidden" id="annotary-form-collection" name="collection" />').attr('value', collectionId)
    );

  $('.annotary-collection-create input').css('background-image', 'url(' + window.browser.getURL('/shared/img/icon_search.png') + ')');
  $('.annotary-collection-create').append($('<img class="annotary-icon-organization" title="Organization Collection" alt="organization" />').attr('src', window.browser.getURL('/shared/img/icon_organization.png')).hide());
  $('.annotary-collection-create').append($('<img class="annotary-icon-private" title="Private Collection" alt="private" />').attr('src', window.browser.getURL('/shared/img/icon_private.png')).hide());
  $('.annotary-collection-create').append($('<img class="annotary-icon-public" title="Public Collection" alt="public" />').attr('src', window.browser.getURL('/shared/img/icon_public.png')).hide());

  // Make two simultaneous API calls - when they're both done, populate the collections picker
  window.api.request({type: 'GET', uri: '/organization.json'}, function (obj) {
    return function (organization) {
      obj.organization = organization;
      if (--obj.apiCallsLeft === 0)
        obj.init();
    }
  }(this), function (obj) {
    return function (errors) {
      obj.organization = null;
      if (--obj.apiCallsLeft === 0)
        obj.init();
    }
  }(this));

  window.api.request({type: 'GET', uri: '/collections.json'}, function (obj) {
    return function (collections) {
      obj.collections = collections;
      if (--obj.apiCallsLeft === 0)
        obj.init();
    }
  }(this));

}

CollectionsPicker.prototype = {

  default_privacy: null,
  collectionId: null,
  organization: null,
  collections: null,
  apiCallsLeft: 2,

  init: function () {

    // Set the default create collection icon
    this.default_privacy = this.organization ? this.organization.default_collection_privacy : 'public';

    // Determine whether to show the owner of each collection
    var hideOwner = true;
    var lastOwner = null;
    for (i in this.collections) {
      if (lastOwner && lastOwner != this.collections[i].owner_id) {
        hideOwner = false;
        break;
      }
      lastOwner = this.collections[i].owner_id;
    }

    // Add the create link to the collection options
    $('.annotary-collection-options').append(
      '<div class="annotary-create-link">' +
        '<a class="annotary-button-round">' +
          '<img alt="Create Collection" />' +
          '<span class="annotary-create-link-text">Create Collection</span>' +
        '</a>' +
      '</div>'
    );
    $('.annotary-create-link img').attr('src', window.browser.getURL('/shared/img/icon_create_collection.png'));

    // Populate the collection form options
    for (i in this.collections) {
      this.addOption(this.collections[i], false, hideOwner);
    }

    // Populate the working collection
    if (this.collectionId) {

      var collection = window.api.objectById(this.collections, this.collectionId);
      this.selectCollection(collection, hideOwner);

    }
    else {

      window.messenger.sendRequest({type: 'data', inst: 'backgroundToolbar', attr: 'workingCollectionId'}, function(obj) {
        return function (response) {
          var collection = window.api.objectById(obj.collections, response);
          obj.selectCollection(collection, hideOwner);
        }
      }(this));

    }

    // Set up events for the form
    $('.annotary-collection').click(function (obj) {
      return function () {
        obj.expand();
      }
    }(this));

    $('.annotary-collection-create input').focus(function (obj) {
      return function () {
        $(this).css({
          'color':'#333333',
          'font-weight':'normal'
        });
        if ($(this).val() == 'Filter or Create' || $(this).val() == '') {
          $(this).val('');
        }
      }
    }(this));

    $('.annotary-collection-create input').blur(function (obj) {
      return function () {
        if ($(this).val() == 'Filter or Create' || $(this).val() == '') {
          $(this).css({
            'color':'#cccccc',
            'font-weight':'bold'
          });
          $('.annotary-collection-options .annotary-option').show();
          $(this).val('Filter or Create');
        }
      }
    }(this));

    $('.annotary-create-link .annotary-button-round').click(function (obj) {
      return function () {
        obj.openCreateCollectionModal();
      }
    }(this));

    $('.annotary-collection-create input').keyup(function (obj) {
      return function (e) {

        if (e.which == 13) {
          obj.openCreateCollectionModal();
          e.preventDefault();
        }
        else {
          var input_val = $('.annotary-collection-create input').val()
          $('.annotary-collection-options .annotary-option').each(function () {
            if ($(this).find('.annotary-option-title').text().toLowerCase().indexOf(input_val.toLowerCase()) > -1)
              $(this).show();
            else
              $(this).hide();
          });
          if (input_val != '')
            $('.annotary-create-link-text').text('Create "' + input_val + '"');
          else
            $('.annotary-create-link-text').text('Create Collection');
        }

      }
    }(this));

    this.bindTooltips();

  },

  expand: function () {

    $('.annotary-collection').hide();
    $('.annotary-collection-expanded').show();

  },

  openCreateCollectionModal: function () {

    $('body').append(
      '<div id="annotary-modal"><div class="annotary-modal-horizon">' +
        '<div class="annotary-modal-content">' +
          '<div class="annotary-modal-header">Create Collection</div>' +
          '<div class="annotary-modal-body">' +
            '<label for="annotary-create-collection-modal-title">Title:</label>' +
            '<input type="text" class="annotary-input-text" id="annotary-create-collection-modal-title" />' +
            '<label for="annotary-create-collection-modal-description">Description:</label>' +
            '<textarea id="annotary-create-collection-modal-description"></textarea>' +
            '<table class="annotary-radio-block">' +
              '<tr>' +
                '<td>' +
                  '<input type="radio" id="annotary-create-collection-modal-privacy-public" name="annotary-privacy" value="public" />' +
                '</td>' +
                '<td>' +
                  '<label for="annotary-create-collection-modal-privacy-public">' +
                    '<img alt="Public" />Public' +
                    '<span class="annotary-desc">Share your bookmarks with all of Annotary.</span>' +
                  '</label>' +
                '</td>' +
              '</tr>' +
              '<tr>' +
                '<td>' +
                  '<input type="radio" id="annotary-create-collection-modal-privacy-private" name="annotary-privacy" value="private" />' +
                '</td>' +
                '<td>' +
                  '<label for="annotary-create-collection-modal-privacy-private">' +
                    '<img alt="Private" />Private' +
                    '<span class="annotary-desc">Only invited collection members may view your bookmarks.</span>' +
                  '</label>' +
                '</td>' +
              '</tr>' +
            '</table>' +
            '<a class="annotary-button">Create Collection</button>' +
            '<a id="annotary-modal-button-cancel">Cancel</a>' +
            '<div id="annotary-modal-loading"></div>' +
          '</div>' +
        '</div>' +
      '</div></div>'
    );
    $('#annotary-modal-loading').append($('<img alt="Saving collection..." />').attr('src', window.browser.getURL('/shared/img/loading.gif')));

    // Add organization privacy if applicable
    if (this.organization) {
      $('#annotary-modal .annotary-radio-block tr:first').after(
        '<tr>' +
          '<td>' +
            '<input type="radio" id="annotary-create-collection-modal-privacy-organization" name="annotary-privacy" value="organization" />' +
          '</td>' +
          '<td>' +
            '<label for="annotary-create-collection-modal-privacy-organization">' +
              '<img alt="Organization" />Organization' +
              '<span class="annotary-desc"></span>' +
            '</label>' +
          '</td>' +
        '</tr>'
      );
      $('label[for=annotary-create-collection-modal-privacy-organization] .annotary-desc').text('Share with everyone in the organization ' + this.organization.title + '.');
    }

    // Add icons to the radio labels
    $('label[for=annotary-create-collection-modal-privacy-public] img')
      .attr('src', window.browser.getURL('/shared/img/icon_public_large.png'))
      .css('margin', '0 5px 0 0');
    $('label[for=annotary-create-collection-modal-privacy-organization] img')
      .attr('src', window.browser.getURL('/shared/img/icon_organization_large.png'))
      .css('margin', '0 6px 0 1px');
    $('label[for=annotary-create-collection-modal-privacy-private] img')
      .attr('src', window.browser.getURL('/shared/img/icon_private_large.png'))
      .css('margin', '0 7px 0 2px');

    // Select the appropriate radio button
    $('#annotary-create-collection-modal-privacy-' + this.default_privacy).prop('checked', true);

    // Position the modal
    var height = $('#annotary-modal .annotary-modal-header').outerHeight() + $('#annotary-modal .annotary-modal-body').outerHeight();
    $('#annotary-modal .annotary-modal-content').css('margin-top', '-' + Math.floor(height/2) + 'px');

    // Populate and focus the title field
    var input_val = $('.annotary-collection-create input').val();
    if (input_val == '' || input_val == 'Filter or Create')
      $('#annotary-create-collection-modal-title').val('').focus();
    else
      $('#annotary-create-collection-modal-title').val(input_val).focus();

    // Close modal on click of cancel button or transparent background
    $('#annotary-modal-button-cancel').live('click', function (e) {
      $('#annotary-modal').remove();
      e.preventDefault();
    });
    $('#annotary-modal').live('click', function () {
      $(this).remove();
    });
    $('#annotary-modal > *').live('click', function (e) {
      e.stopPropagation();
    });

    // Create collection on click of button
    $('#annotary-modal .annotary-button').click(function (obj) {
      return function () {

        $('#annotary-modal-loading').show();
        $('#annotary-modal .annotary-button').hide();
        $('#annotary-modal-button-cancel').hide();
        window.api.request({
          type: 'POST',
          uri: '/collections.json',
          data: {
            'title': $('#annotary-create-collection-modal-title').val(),
            'description': $('#annotary-create-collection-modal-description').val(),
            'privacy': $('#annotary-modal input[name=annotary-privacy]:checked').val()
          }
        }, function (obj) {
          return function (collection) {
            $('#annotary-modal').remove();
            var hideOwner = $('.annotary-option-owner').length == 0;
            obj.addOption(collection, true, hideOwner);
            obj.selectCollection(collection, hideOwner);
            $('.annotary-collection-create input').val('').blur();
          }
        }(obj), function (errors) {
          $('#annotary-modal-loading').hide();
          $('#annotary-modal .annotary-button').show();
          $('#annotary-modal-button-cancel').show();
          $('#annotary-create-collection-modal-title').css('background', '#ffcccc').css('border', '1px solid #ff0000');
        });

      }
    }(this));

  },

  selectCollection: function (collection, hideOwner) {

    if (typeof collection == 'undefined')
      this.expand();
    else {

      window.messenger.sendRequest({type: 'func', inst: 'backgroundToolbar', func: 'setWorkingCollectionId', args: [collection.id]});

      $('.annotary-collection')
        .html('')
        .append($('<div class="annotary-option-title"></div>').text(collection.title));

      if (collection.privacy == 'private') {
        $('.annotary-collection')
          .addClass('annotary-option-private')
          .prepend($('<img class="annotary-icon-private" data-tooltip="left" title="Private Collection" alt="private" />').attr('src', window.browser.getURL('/shared/img/icon_private.png')));
      }
      else if (collection.privacy == 'organization') {
        $('.annotary-collection')
          .addClass('annotary-option-private')
          .prepend($('<img class="annotary-icon-organization" data-tooltip="left" title="Organization Collection" alt="organization" />').attr('src', window.browser.getURL('/shared/img/icon_organization.png')));
      }
      else {
        $('.annotary-collection')
          .removeClass('annotary-option-private')
          .prepend($('<img class="annotary-icon-public" data-tooltip="left" title="Public Collection" alt="public" />').attr('src', window.browser.getURL('/shared/img/icon_public.png')));
      }

      if (!hideOwner) {
        $('.annotary-collection')
          .removeClass('annotary-option-no-owner')
          .append($('<div class="annotary-option-owner"></div>').text(collection.owner_display_name));
      }
      else
        $('.annotary-collection').addClass('annotary-option-no-owner');

      $('.annotary-collection')
        .append($('<img class="annotary-icon-dropdown" />').attr('src', window.browser.getURL('/shared/img/icon_dropdown.png')))
        .show();

      $('#annotary-form-collection').val(collection.id);
      $('.annotary-collection-expanded').hide();

      $('.annotary-collection-create input').val('').blur();

      this.bindTooltips();

    }

  },

  addOption: function (collection, before, hideOwner) {

    var option = $('<div class="annotary-option"></div>')
      .attr('id', 'annotary-option-' + collection.id)
      .append(
        $('<div class="annotary-option-title"></div>').text(collection.title)
      );

    if (collection.privacy == 'private') {
      option
        .addClass('annotary-option-private')
        .prepend($('<img class="annotary-icon-private" data-tooltip="left" title="Private Collection" alt="private" />').attr('src', window.browser.getURL('/shared/img/icon_private.png')));
    }
    else if (collection.privacy == 'organization') {
      option
        .addClass('annotary-option-private')
        .prepend($('<img class="annotary-icon-organization" data-tooltip="left" title="Organization Collection" alt="organization" />').attr('src', window.browser.getURL('/shared/img/icon_organization.png')));
    }
    else {
      option
        .removeClass('annotary-option-private')
        .prepend($('<img class="annotary-icon-public" data-tooltip="left" title="Public Collection" alt="public" />').attr('src', window.browser.getURL('/shared/img/icon_public.png')));
    }

    if (!hideOwner) {
      option
        .removeClass('annotary-option-no-owner')
        .append($('<div class="annotary-option-owner"></div>').text(collection.owner_display_name));
    }
    else
      option.addClass('annotary-option-no-owner');

    if (before)
      $('.annotary-collection-options').prepend(option);
    else
      $('.annotary-create-link').before(option);

    $('#annotary-option-' + collection.id).click(function (obj, collection, hideOwner) {
      return function () {
        obj.selectCollection(collection, hideOwner);
      }
    }(this, collection, hideOwner));

  },

  bindTooltips: function () {

    $('#annotary-toolbar-form [title][data-tooltip=top], #annotary-toolbar-form [title]:not([data-tooltip])').tooltip({
      show: false,
      hide: false,
      tooltipClass: "annotary-ui-tooltip",
      position: {
        my: "center bottom-15",
        at: "center top"
      },
      open: function (e, ui) {
        ui.tooltip.addClass('annotary-ui-tooltip-arrow').addClass('annotary-ui-tooltip-top');
      }
    });

    $('#annotary-toolbar-form [title][data-tooltip=left]').tooltip({
      show: false,
      hide: false,
      tooltipClass: "annotary-ui-tooltip",
      position: {
        my: "right-15 center",
        at: "left center"
      },
      open: function (e, ui) {
        ui.tooltip.addClass('annotary-ui-tooltip-arrow').addClass('annotary-ui-tooltip-left');
      }
    });

  }

}
