ContentToolbar = function () {

  // Set up toolbar events
  $('#annotary-toolbar-link-highlight').live('mousedown', function (obj) {
    return function () {
      if (obj.currentMode != 'network') // Don't save selection if someone else's highlights are shown
        obj.saveSelection(); // Save the selection to be highlighted when highlight mode comes on in this tab
      obj.toggleHighlightMode();
    }
  }(this));
  $('#annotary-toolbar-link-bookmark').live('mousedown', function (obj) {
    return function () {
      obj.toggleBookmarkMode();
    }
  }(this));
  $('#annotary-toolbar-link-share').live('mousedown', function (obj) {
    return function () {
      obj.toggleShareMode();
    }
  }(this));
  $('#annotary-toolbar-link-users').live('mousedown', function (obj) {
    return function () {
      obj.toggleNetworkMode();
    }
  }(this));
  $('#annotary-toolbar-close').live('click', function (obj) {
    return function () {
      window.messenger.sendRequest({type: 'func', inst: 'backgroundToolbar', func: "close"});
    }
  }(this));

  // Add a new css style to make the cursor the highlighter when in highlight mode
  $("<style type='text/css'> body.annotary-highlight-mode { cursor: url('" + window.browser.getURL('/shared/img/cursor_highlight.png') + "') 4 11,auto !important; } </style>").appendTo("head");

  // If the toolbar is open in the background script, open the toolbar here
  window.messenger.sendRequest({type: 'data', inst: 'backgroundToolbar', attr: 'isOpen'}, function(obj) {
    return function (response) {
      if (response == true)
        obj.open();
    }
  }(this));

}

ContentToolbar.prototype = {

  isOpen: null,
  currentMode: null,
  savedSelection: null,

  open: function () {

    if (this.isOpen == true)
      return;

    $('body').append('<div id="annotary-toolbar">' +
      '<div id="annotary-toolbar-main">' +
        '<div id="annotary-toolbar-bg"></div>' +
        '<div id="annotary-toolbar-menubar" class="annotary-menubar">' +
          '<a id="annotary-toolbar-close" class="annotary-menubar-close"></a>' +
        '</div>' +
        '<div id="annotary-toolbar-nav">' +
          '<a title="Bookmark Page" id="annotary-toolbar-link-bookmark" class="annotary-toolbar-link"></a>' +
          '<div class="annotary-toolbar-seperator"></div>' +
          '<a title="Highlight Page" id="annotary-toolbar-link-highlight" class="annotary-toolbar-link"></a>' +
          '<div class="annotary-toolbar-seperator"></div>' +
          '<a title="Share Highlighted Page" id="annotary-toolbar-link-share" class="annotary-toolbar-link"></a>' +
          '<div class="annotary-toolbar-seperator"></div>' +
          '<a title="Community Bookmarks &amp; Highlights" id="annotary-toolbar-link-users" class="annotary-toolbar-link"></a>' +
          '<div class="annotary-toolbar-seperator"></div>' +
          '<a title="Annotary" id="annotary-toolbar-link-home" class="annotary-toolbar-link" target="_blank"></a>' +
        '</div>' +
      '</div>' +
      '<div id="annotary-toolbar-form"></div>' +
    '</div>');

    $('#annotary-toolbar-form').hide();

    this.setBookmarkIndicators();

    $('#annotary-toolbar-close')
      .css('background', 'url(' + window.browser.getURL('/shared/img/btn_close.png') + ')')
      .hover(function () {
        $(this).css('background', 'url(' + window.browser.getURL('/shared/img/btn_close_on.png') + ')');
      }, function () {
        $(this).css('background', 'url(' + window.browser.getURL('/shared/img/btn_close.png') + ')');
      });

    $('#annotary-toolbar-link-home')
      .css('background-image', 'url(' + window.browser.getURL('/shared/img/icon_home.png') + ')')
      .click(function() {
        // Manually fire mouseoff event then open the window
        $(this).css('background-image', 'url(' + window.browser.getURL('/shared/img/icon_home.png') + ')');
        if (window.bookmarkRegistry.iHaveBookmarked()) {
          var myBookmark = window.bookmarkRegistry.bookmarks[window.bookmarkRegistry.myBookmark];
          window.open('https://annotary.com/collections/' + myBookmark.collection.id + '/' + myBookmark.collection.slug + '?utm_source=' + document.domain + '&utm_medium=extension');
        }
        else
          window.open('https://annotary.com/profiles/me?utm_source=' + document.domain + '&utm_medium=extension');
      });

    $('#annotary-toolbar').draggable({
      handle: '#annotary-toolbar-menubar',
      start: function () {
        $(this).css('right', 'auto');
      }
    });

    this.resetLinks();
    this.isOpen = true;

    // If we're already showing someone else's highlights, open this thing up in network mode
    if (window.bookmarkRegistry.showingSomeoneElsesBookmark())
      this.toggleNetworkMode(true);

    this.bindTooltips();

  },

  close: function () {

    $('body').removeClass('annotary-highlight-mode');
    $('#annotary-toolbar').remove();
    this.isOpen = false;
    this.currentMode = null;

  },

  setBookmarkIndicators: function () {

    $('#annotary-toolbar-link-users > span').remove();
    $('#annotary-toolbar-link-bookmark > span').remove();

    var count = window.bookmarkRegistry.countBookmarks() + window.bookmarkRegistry.countDomainBookmarks();

    if (count > 0) {
      $('#annotary-toolbar-link-users').append(
        $('<span></span>').text(count)
      );
    }

    if (window.bookmarkRegistry.iHaveBookmarked())
      $('#annotary-toolbar-link-bookmark').append('<span>&#10004;</span>');

  },

  saveSelection: function () {

    var sel = rangy.getSelection();
    if (!sel.isCollapsed) {
      this.savedSelection = rangy.saveSelection();
      sel.collapseToEnd();
    }

  },

  highlightSavedSelection: function () {

    if (this.savedSelection) {
      var sel = rangy.getSelection();
      sel.collapseToEnd();
      rangy.restoreSelection(this.savedSelection);
      $('.rangySelectionBoundary').remove();
      window.highlighter.createHighlightFromSelection();
    }

  },

  deleteSavedSelection: function () {

    this.savedSelection = null;

  },

  toggleHighlightMode: function () {

    if (this.isOpen == false)
      return;

    if (this.currentMode == 'highlight') {
      this.nullMode();
      return;
    }

    this.currentMode = 'highlight';
    this.resetLinks();

    $('body').addClass('annotary-highlight-mode');
    $('#annotary-toolbar-link-highlight')
      .css('background-image', 'url(' + window.browser.getURL('/shared/img/icon_highlight_on.png') + ')')
      .addClass('annotary-toolbar-link-on');
    $('#annotary-toolbar-form').hide();

    // Switch to myBookmark, then if there was a selection waiting to be highlighted, highlight it
    window.bookmarkRegistry.loadBookmark(window.bookmarkRegistry.myBookmark, function (obj) {
      return function () {
        obj.highlightSavedSelection();
      }
    }(this));

  },

  toggleNetworkMode: function (onStateOnly) {

    if (this.isOpen == false)
      return;

    // Go to null mode if we're in network mode
    if (this.currentMode == 'network' && $('#annotary-toolbar-form').is(':visible')) {
      this.nullMode();
      return;
    }

    $('body').removeClass('annotary-highlight-mode');
    this.currentMode = 'network';
    this.resetLinks();
    $('#annotary-toolbar-link-users')
      .css('background-image', 'url(' + window.browser.getURL('/shared/img/icon_users_on.png') + ')')
      .addClass('annotary-toolbar-link-on');

    // If it's not just setting the toolbar, open the menu of users
    if (!onStateOnly) {

      $('#annotary-toolbar-form').html(
        '<a id="annotary-button-closeform">&raquo;</a>' +
        '<div id="annotary-toolbar-form-content"></div>'
      );

      var count = window.bookmarkRegistry.countBookmarks();
      if (count > 0) {
        $('#annotary-toolbar-form').prepend(
          '<a id="annotary-toolbar-form-tab-page" class="annotary-toolbar-form-tab"></a>' +
          '<a id="annotary-toolbar-form-tab-domain" class="annotary-toolbar-form-tab"></a>' +
          '<div style="clear:both;"></div>'
        );
        $('#annotary-toolbar-form-tab-page').text('This Page');
        $('#annotary-toolbar-form-tab-domain').text('Complete Site');
        this.networkModeHighlights();
      }
      else {
        this.networkModeBookmarks();
      }

      $('#annotary-toolbar-form').show();

      // Set up events
      $('#annotary-button-closeform').click(function (obj) {
        return function () {
          obj.nullMode();
        }
      }(this));
      $('#annotary-toolbar-form-tab-page').click(function (obj) {
        return function () {
          obj.networkModeHighlights();
        }
      }(this));
      $('#annotary-toolbar-form-tab-domain').click(function (obj) {
        return function () {
          obj.networkModeBookmarks();
        }
      }(this));

    }

  },

  networkModeHighlights: function () {

    $('#annotary-toolbar-form-content').html('<div id="annotary-toolbar-form-heading">This page bookmarked by:</div>');
    $('#annotary-toolbar-form-tab-page').addClass('annotary-toolbar-form-tab-active');
    $('#annotary-toolbar-form-tab-domain').removeClass('annotary-toolbar-form-tab-active');

    for (id in window.bookmarkRegistry.bookmarks) {
      if (id == window.bookmarkRegistry.myBookmark)
        continue;
      var bookmark = window.bookmarkRegistry.bookmarks[id];
      $('#annotary-toolbar-form-content').append(
        $('<a class="annotary-bookmark"></a>')
          .attr('data-id', id)
          .append(
            $('<img class="annotary-bookmark-thumb" />')
              .attr('src', bookmark.user.profile.image)
              .attr('alt', bookmark.user.first_name + ' ' + bookmark.user.last_name)
          )
          .append(
            $('<span class="annotary-bookmark-info"></span>')
              .append(
                $('<span class="annotary-bookmark-name"></span>').text(bookmark.user.first_name + ' ' + bookmark.user.last_name)
              )
              .append(
                $('<span class="annotary-bookmark-highlights"></span>').text(bookmark.highlights_count + ' highlight' + (bookmark.highlights_count == 1 ? '' : 's'))
              )
          )
          .append(
            $('<div style="clear:both"></div>')
          )
      );
    }

    $('a.annotary-bookmark').click(function (obj) {
      return function () {
        var id = $(this).attr('data-id');
        window.bookmarkRegistry.loadBookmark(id);
        $('#annotary-toolbar-form').hide();
      }
    }(this));

  },

  networkModeBookmarks: function () {

    $('#annotary-toolbar-form-content').html('')
    $('#annotary-toolbar-form-tab-page').removeClass('annotary-toolbar-form-tab-active');
    $('#annotary-toolbar-form-tab-domain').addClass('annotary-toolbar-form-tab-active');

    var my_count = window.bookmarkRegistry.countDomainBookmarks('my');
    var other_count = window.bookmarkRegistry.countDomainBookmarks('other');
    var domain = document.domain.indexOf('www.') == 0 ? document.domain.slice(4) : document.domain;

    if (my_count > 0) {
      $('#annotary-toolbar-form-content').append(
        $('<div id="annotary-toolbar-form-heading"></div>').text('My bookmarks on ' + domain + ':')
      );
      for (id in window.bookmarkRegistry.my_domain_bookmarks) {
        var bookmark = window.bookmarkRegistry.my_domain_bookmarks[id];
        this.appendBookmarkToForm(bookmark);
      }
    }

    if (other_count > 0) {
      $('#annotary-toolbar-form-content').append(
        $('<div id="annotary-toolbar-form-heading"></div>')
          .css('margin-top', (my_count > 0 ? '15px' : '0'))
          .text('Bookmarks from my feed on ' + domain + ':')
      );
      for (id in window.bookmarkRegistry.other_domain_bookmarks) {
        var bookmark = window.bookmarkRegistry.other_domain_bookmarks[id];
        this.appendBookmarkToForm(bookmark, true);
      }
    }

    if (my_count == 0 && other_count == 0) {
      $('#annotary-toolbar-form-content').append(
        $('<div id="annotary-toolbar-form-heading"></div>').text('Bookmarks from my feed on ' + domain + ':')
      ).append(
        $('<div id="annotary-no-highlights"></div>').text('No bookmarks yet on ' + domain)
      );
      if (!window.bookmarkRegistry.iHaveBookmarked())
        $('#annotary-toolbar-form-content').append('<br /><a id="annotary-goto-bookmark-mode">Bookmark this page</a>');
      $('#annotary-goto-bookmark-mode').click(function (obj) {
        return function () {
          obj.toggleBookmarkMode();
        }
      }(this));
    }

  },

  appendBookmarkToForm: function (bookmark, showUser) {

    var div = $('<div class="annotary-domain-bookmark-info"></div>');

    div.append(
      $('<a class="annotary-bookmark-title" target="_blank"></a>')
        .attr('href', 'http://annotary.com/bookmarks/' + bookmark.id + '/' + bookmark.slug)
        .text(bookmark.title)
    );

    if (showUser) {
      div.append(' by ').append(
        $('<a target="_blank"></a>')
          .attr('href', 'https://annotary.com/profiles/' + bookmark.user.username)
          .text(bookmark.user.first_name + ' ' + bookmark.user.last_name)
      ).append('<br />');
    }

    div.append(' from ').append(
      $('<a target="_blank"></a>')
        .attr('href', 'https://annotary.com/collections/' + bookmark.collection.id + '/' + bookmark.collection.slug)
        .text(bookmark.collection.title)
    );

    $('#annotary-toolbar-form-content').append(
      $('<div class="annotary-domain-bookmark"></div>').append(
        $('<a class="annotary-domain-bookmark-image" target="_blank"></a>')
          .attr('href', 'http://annotary.com/bookmarks/' + bookmark.id + '/' + bookmark.slug)
          .append(
            $('<img />').attr({
              'src' : $.trim(bookmark.image) ? 'http://media.annotary.com/' + $.trim(bookmark.image) : window.browser.getURL('/shared/img/icon_bookmark_default.png'),
              'alt' : bookmark.title,
            })
          )
      ).append(div).append('<div style="clear:both;"></div>')
    );

  },

  toggleShareMode: function () {

    if (this.isOpen == false)
      return;

    // Go to null mode if we're in share mode already
    if (this.currentMode == 'share') {
      this.nullMode();
      return;
    }

    // If the page hasn't been bookmarked, go to Bookmark mode
    if (!window.bookmarkRegistry.iHaveBookmarked()) {
      window.contentToolbar.toggleBookmarkMode('Save the page to share highlights.', true);
      return;
    }

    // Go back to mybookmark if in network mode
    if (this.currentMode == 'network')
      window.bookmarkRegistry.loadBookmark(window.bookmarkRegistry.myBookmark);

    $('body').removeClass('annotary-highlight-mode');
    this.currentMode = 'share';
    this.resetLinks();
    $('#annotary-toolbar-link-share')
      .css('background-image', 'url(' + window.browser.getURL('/shared/img/icon_share_on.png') + ')')
      .addClass('annotary-toolbar-link-on');

    // Open the share menu
    $('#annotary-toolbar-form').html(
      '<div id="annotary-toolbar-form-heading">Share Your Highlighted Page</div>' +
      '<a id="annotary-button-closeform">&raquo;</a>' +
      '<div id="annotary-toolbar-form-message" style="display:none;margin-top:10px;"></div>' +
      '<input type="text" id="annotary-share-permalink" readonly="readonly" />'
    );

    // Set icons and permalink to link to the highlighted page
    var bookmark = window.bookmarkRegistry.bookmarks[window.bookmarkRegistry.myBookmark];
    var url = 'http://annotary.com/bookmarks/' + bookmark.id + '/' + bookmark.slug;
    $('#annotary-share-permalink').val(url).click(function () { $(this).select(); })

    if (bookmark.collection.privacy == 'public' || bookmark.collection.visible_with_link) {
      $('#annotary-toolbar-form').append(
        '<label class="annotary-center">Social Networks</label>' +
        '<div id="annotary-social-networks" class="annotary-center"></div>' +
        '<label class="annotary-center">Email</label>' +
        '<div id="annotary-email-clients" class="annotary-center"></div>'
      );
      var networks = {
        'facebook': {alt:'Facebook', container:'#annotary-social-networks'},
        'twitter': {alt:'Twitter', container:'#annotary-social-networks'},
        'google_plusone_share': {alt:'Google+', container:'#annotary-social-networks'},
        'linkedin': {alt:'LinkedIn', container:'#annotary-social-networks'},
        'gmail': {alt:'GMail', container:'#annotary-email-clients'},
        'yahoomail': {alt:'Yahoo! Mail', container:'#annotary-email-clients'},
        'hotmail': {alt:'Hotmail', container:'#annotary-email-clients'},
        'aolmail': {alt:'AOL Mail', container:'#annotary-email-clients'},
      }
    }
    else {
      $('#annotary-toolbar-form').append(
        '<label class="annotary-center">Email</label>' +
        '<div id="annotary-email-clients" class="annotary-center"></div>'
      );
      var networks = {
        'gmail': {alt:'GMail', container:'#annotary-email-clients'},
        'yahoomail': {alt:'Yahoo! Mail', container:'#annotary-email-clients'},
        'hotmail': {alt:'Hotmail', container:'#annotary-email-clients'},
        'aolmail': {alt:'AOL Mail', container:'#annotary-email-clients'},
      }
      if (bookmark.collection.privacy == 'organization')
        $('#annotary-toolbar-form-message').text('This bookmark is in an organization collection. Only members of the organization can view it when shared.').show();
      else
        $('#annotary-toolbar-form-message').text('This bookmark is in a private collection. Only members of the collection can view it when shared.').show();
    }

    for (var network in networks) {
      var network_url = 'http://www.addthis.com/bookmark.php?v=250&pub=ra-4fd0f1e37e16667f&ct=1&s=' + network + '&title=' + bookmark.title + '&url=' + encodeURIComponent(url);
      $(networks[network].container).append(
        $('<a class="annotary-social-icon"></a>')
          .attr('href', network_url)
          .append(
            $('<img />')
              .attr('src', 'http://cache.addthiscdn.com/icons/v1/thumbs/32x32/' + network + '.png')
              .attr('alt', networks[network].alt)
              .attr('title', networks[network].alt)
          ).click(function () {
            var top = (screen.height/2) - 200;
            var left = (screen.width/2) - 275;
            window.open($(this).attr('href'), 'annotary-share-window', 'status=0,toolbar=0,directories=0,height=400,width=550,top=' + top + ',left=' + left);
            return false;
          })
      );
    }

    // Enable tooltips
    $('#annotary-toolbar-form [title]').tooltip({
      show: false,
      hide: false,
      tooltipClass: "annotary-ui-tooltip",
      position: {
        my: "center bottom-15",
        at: "center top"
      },
      open: function (e, ui) {
        ui.tooltip.addClass('annotary-ui-tooltip-arrow').addClass('annotary-ui-tooltip-top');
      }
    });

    $('#annotary-toolbar-form').show();

    // Set up events
    $('#annotary-button-closeform').click(function (obj) {
      return function () {
        obj.nullMode();
      }
    }(this));

  },

  toggleBookmarkMode: function (message, saveSelection) {

    if (this.isOpen == false)
      return;

    var prevMode = this.currentMode;

    // Go to null mode if we're in bookmark mode
    if (this.currentMode == 'bookmark') {
      this.nullMode();
      return;
    }

    // Open up the bookmarking form otherwise
    this.currentMode = 'bookmark';
    this.resetLinks();

    $('body').removeClass('annotary-highlight-mode');
    $('#annotary-toolbar-link-bookmark')
      .css('background-image', 'url(' + window.browser.getURL('/shared/img/icon_bookmark_on.png') + ')')
      .addClass('annotary-toolbar-link-on');
    this.setFormContent(message);
    $('#annotary-toolbar-form').show();

    // Go back to mybookmark if in network mode, otherwise save the selection to restore after bookmarking
    if (prevMode == 'network')
      window.bookmarkRegistry.loadBookmark(window.bookmarkRegistry.myBookmark);
    else if (saveSelection)
      this.saveSelection();

  },

  nullMode: function () {

    if (this.isOpen == false || this.currentMode == null)
      return;

    this.currentMode = null;
    this.resetLinks();

    $('body').removeClass('annotary-highlight-mode');
    $('#annotary-toolbar-form').hide();

    this.deleteSavedSelection();

    // Go back to myBookmark if we were in network mode
    window.bookmarkRegistry.loadBookmark(window.bookmarkRegistry.myBookmark);

  },

  resetLinks: function () {

    $('#annotary-toolbar-link-highlight')
      .css('background-image', 'url(' + window.browser.getURL('/shared/img/icon_highlight.png') + ')')
      .removeClass('annotary-toolbar-link-on');

    $('#annotary-toolbar-link-bookmark')
      .css('background-image', 'url(' + window.browser.getURL('/shared/img/icon_bookmark.png') + ')')
      .removeClass('annotary-toolbar-link-on');

    $('#annotary-toolbar-link-users')
      .css('background-image', 'url(' + window.browser.getURL('/shared/img/icon_users.png') + ')')
      .removeClass('annotary-toolbar-link-on');

    $('#annotary-toolbar-link-share')
      .css('background-image', 'url(' + window.browser.getURL('/shared/img/icon_share.png') + ')')
      .removeClass('annotary-toolbar-link-on');

  },

  setFormContent: function (message) {

    // Get the current data to populate the form
    var formData = window.bookmarkRegistry.bookmarks[window.bookmarkRegistry.myBookmark];

    // Set the HTML
    $('#annotary-toolbar-form').html(
      '<label for="annotary-form-title" style="margin-top:0;">Page Title</label>' +
      '<input type="text" class="annotary-form-field" id="annotary-form-title" name="title" />' +
      '<label for="annotary-form-note">Note <span style="font-weight:normal">(optional)</span></label>' +
      '<textarea class="annotary-form-field" id="annotary-form-note" rows="2" name="note"></textarea>' +
      '<label for="annotary-form-collection">Collection</label>' +
      '<div id="annotary-form-collection-error"></div>' +
      '<div id="annotary-collection-picker-container"></div>' +
      '<a id="annotary-button-save" class="annotary-button">Save Page</a>' +
      '<a id="annotary-button-cancel">Cancel</a>' +
      (formData.id ? '<a id="annotary-button-remove">Remove</a>' : '') + 
      '<div id="annotary-form-loading"></div>'
    );

    $('#annotary-form-loading').append($('<img alt="Saving bookmark..." />').attr('src', window.browser.getURL('/shared/img/loading.gif')));

    $('#annotary-form-title').attr('value', formData.title);
    $('#annotary-form-note').text(formData.note);

    // Set the message if there is one
    if (message) {
      $('#annotary-toolbar-form').prepend(
        $('<div id="annotary-toolbar-form-message"></div>').text(message)
      );
    }

    // Set up collections picker
    var picker = new CollectionsPicker(formData.collection.id, $('#annotary-collection-picker-container'));

    // Set up events for the form
    $('#annotary-button-remove')
      .css('background-image', 'url(' + window.browser.getURL('/shared/img/icon_trash_red.png') + ')')
      .click(function (obj) {
        return function () {
          obj.promptRemoveBookmark();
        }
      }(this));
    $('#annotary-button-cancel').click(function (obj) {
      return function () {
        obj.toggleBookmarkMode();
      }
    }(this));
    $('#annotary-button-save').click(function (obj) {
      return function () {
        obj.saveBookmark();
      }
    }(this));

  },

  saveBookmark: function () {

    var bookmark = $.extend(true, {}, window.bookmarkRegistry.bookmarks[window.bookmarkRegistry.myBookmark]);

    // Get the data from the fields, put to the API, then save to the page object on success
    if ($('#annotary-form-title').length > 0) {
      bookmark.title = $('#annotary-form-title').val();
      bookmark.publisher = window.location.href.replace("http://", "").replace("https://", "").split("/")[0],
      bookmark.pub_date = bookmark.pub_date ? bookmark.pub_date : "",
      bookmark.note = $('#annotary-form-note').val();
    }
    bookmark.collection = $('#annotary-form-collection').val();
    bookmark.url = bookmark.url.replace(/ /g, '%20');

    if (bookmark.id)
      var request = {type: 'PUT', uri: '/bookmarks/'+bookmark.id+'.json'}
    else
      var request = {type: 'POST', uri: '/bookmarks.json'}
    request.data = bookmark;
    delete request.data.id;

    $('#annotary-button-save').hide();
    $('#annotary-button-cancel').hide();
    $('#annotary-button-remove').hide();
    $('#annotary-form-loading').show();

    window.api.request(request, function (obj) {
      return function (response) {
        window.bookmarkRegistry.setMyBookmarkData(response);
        obj.toggleHighlightMode();
        obj.setBookmarkIndicators();
      }
    }(this), function (errors) {
      $('.annotary-form-field').css('background', '#ffffff').css('border', '1px solid #cccccc');
      $('#annotary-form-collection-error').hide();
      for (field in errors) {
        if (field == 'collection')
          $('#annotary-form-collection-error').text('Create a collection to save the page').show();
        else
          $('#annotary-form-'+field).css('background', '#ffcccc').css('border', '1px solid #ff0000');
      }
      $('#annotary-button-save').show();
      $('#annotary-button-cancel').show();
      $('#annotary-button-remove').show();
      $('#annotary-form-loading').hide();
    });

  },

  promptRemoveBookmark: function () {

    $('#annotary-toolbar-form').html(
      '<h3>Remove Bookmark</h3>' +
      '<p>Are you sure you want to permanently remove this bookmark? ' +
      'This will also permanently delete all highlights and notes on this page. ' +
      '<b>There is no undo.</b></p>' +
      '<a id="annotary-button-delete" class="annotary-button">Remove Bookmark</a>' +
      '<a id="annotary-button-cancel">Cancel</a>' + 
      '<div id="annotary-form-loading"></div>'
    );

    $('#annotary-form-loading').append($('<img alt="Deleting bookmark..." />').attr('src', window.browser.getURL('/shared/img/loading.gif')));

    $('#annotary-button-cancel').click(function (obj) {
      return function () {
        obj.toggleBookmarkMode();
      }
    }(this));
    $('#annotary-button-delete').click(function (obj) {
      return function () {
        obj.removeBookmark();
      }
    }(this));

  },

  removeBookmark: function () {

    var bookmarkId = window.bookmarkRegistry.myBookmark;

    $('#annotary-button-delete').hide();
    $('#annotary-button-cancel').hide();
    $('#annotary-form-loading').show();

    window.api.request({type: 'DELETE', uri: '/bookmarks/'+bookmarkId+'.json'}, function (obj) {
      return function (response) {
        obj.toggleBookmarkMode();
        window.location.reload();
      }
    }(this));

  },

  bindTooltips: function () {

    $('#annotary-toolbar-nav [title]').tooltip({
      show: {duration: 0, delay: 2000},
      hide: false,
      tooltipClass: "annotary-ui-tooltip",
      position: {
        my: "right-15 center",
        at: "left center"
      },
      open: function (e, ui) {
        ui.tooltip.addClass('annotary-ui-tooltip-arrow').addClass('annotary-ui-tooltip-left');
      }
    });

  }

}
