ContentHighlighter = function () {

  // Enable highlighting event
  $(document).mouseup(function(obj) {
    return function () {
      obj.createHighlightFromSelection();
    }
  }(this));

  // Close the note window on click outside of it
  $(document).mousedown(function (obj) {
    return function () {
      if (obj.noteOpen)
        obj.closeNoteWindow();
    }
  }(this));

  // Init rangy
  rangy.init();

}

ContentHighlighter.prototype = {

  noteOpen: null, // Which highlight has it's note window open
  noteSaveTimeout: null, // Timeout object for auto-saving the notes taken on a highlight
  hoveredHighlight: null, // Which highlight I'm hovering over now
  nextTempId: 0, // Next temporary ID for highlights that have been sent to the API as created, but not returned yet

  createHighlightFromSelection: function (hasTimedOut) {

    var sel = rangy.getSelection();
    var ranges = sel.getAllRanges();

    if (ranges.length == 0)
      return;

    var range = sel.getRangeAt(0);
    var rangeStr = range.toString();
    var rangeHtml = range.toHtml();

    if (window.contentToolbar.currentMode != 'highlight' || sel.isCollapsed || rangeStr.match(/^\s*$/))
      return;

    // If it's just a single alphanumeric word, this could be a double-click selection. Wait 300ms and try again in case it's a triple-click.
    if (!hasTimedOut && rangeStr.match(/^[a-z0-9]+$/i)) {
      setTimeout(function (obj) {
        return function () {
          obj.createHighlightFromSelection(true);
        }
      }(this), 300);
      return;
    }

    if (rangeStr.length > 2000) {
      alert('Highlights must be less than 2000 characters.');
      return;
    }

    if (rangeHtml.indexOf('annotary-highlight') > -1 || range.commonAncestorContainer.parentElement.className.indexOf('annotary-highlight') > -1) {
      alert('Highlights cannot overlap.');
      return;
    }

    // If the page isn't bookmarked, open the bookmark window instead, saving the selection to be highlighted on save of the bookmark
    if (!window.bookmarkRegistry.iHaveBookmarked()) {
      window.contentToolbar.toggleBookmarkMode('Save the page to start highlighting.', true);
      return;
    }

    // Compile information to serialize and save highlight to the server
    var savedRange = this.saveRange(range);
    var postData = {
      bookmark: window.bookmarkRegistry.myBookmark,
      html: rangeHtml,
      serialized: JSON.stringify(savedRange),
      location_index: $('*').index(range.startContainer.parentElement),
      location_offset: savedRange.startPoint.offset
    }

    // Create a highlight with a temporary ID
    var tempHighlight = {
      id: 'temp' + this.nextTempId++,
      serialized: postData.serialized,
      note: '',
    };
    this.highlightRange(range, tempHighlight);
    sel.collapseToEnd();
    this.openNoteWindow(tempHighlight.id);

    window.api.request({type: 'POST', uri: '/highlights.json', data: postData}, function (obj, tempHighlight) {
      return function (response) {

        // Change the classname of the .annotary-highlight-id- .annotary-highlight-note- .annotary-highlight-remove-
        $('.annotary-highlight-id-' + tempHighlight.id).removeClass('annotary-highlight-id-' + tempHighlight.id).addClass('annotary-highlight-id-' + response.id);
        $('.annotary-highlight-note-' + tempHighlight.id).removeClass('annotary-highlight-note-' + tempHighlight.id).addClass('annotary-highlight-note-' + response.id);
        $('.annotary-highlight-remove-' + tempHighlight.id).removeClass('annotary-highlight-remove-' + tempHighlight.id).addClass('annotary-highlight-remove-' + response.id);

        // If the note window is open for the temp highlight, change noteOpen to the right ID
        if (obj.noteOpen == tempHighlight.id)
          obj.noteOpen = response.id;

        // Change the id of the highlight in the bookmark registry for currenthighlights
        window.bookmarkRegistry.currentHighlights[response.id] = window.bookmarkRegistry.currentHighlights[tempHighlight.id];
        window.bookmarkRegistry.currentHighlights[response.id].id = response.id;
        delete window.bookmarkRegistry.currentHighlights[tempHighlight.id];

        // Reset click events for the highlight and its icons
        obj.setHighlightClickEvent(response.id);
        obj.setRemoveIcon(response.id);
        obj.setNoteIcon(response.id);

      }
    }(this, tempHighlight));

  },

  highlightRange: function (range, highlight) {

    window.bookmarkRegistry.addHighlightData(highlight, range);
    var highlightApplier = rangy.createCssClassApplier('annotary-highlight-id-' + highlight.id, true);
    highlightApplier.applyToRange(range);

    $('.annotary-highlight-id-' + highlight.id).addClass('annotary-highlight');
    this.setHighlightClickEvent(highlight.id);

    if (highlight.note || window.bookmarkRegistry.myBookmark == window.bookmarkRegistry.currentBookmark)
      $('.annotary-highlight-id-' + highlight.id).addClass('annotary-highlight-clickable');

    this.setNoteIcon(highlight.id);
    this.setRemoveIcon(highlight.id);

  },

  setHighlightClickEvent: function (id) {

    $('.annotary-highlight-id-' + id).unbind('click').click(function (obj, id) {
      return function (event) {
        obj.openNoteWindow(id);
        event.stopPropagation();
      }
    }(this, id));

  },

  openNoteWindow: function (id) {

    if (this.noteOpen != null)
      this.closeNoteWindow();

    if (window.bookmarkRegistry.currentBookmark == window.bookmarkRegistry.myBookmark) {
      var html = '<div id="annotary-note-form">' +
        '<textarea id="annotary-note-content" name="note"></textarea>' +
      '</div>'
    }
    else if (window.bookmarkRegistry.currentHighlights[id].note) {
      var html = '<div id="annotary-note-form">' +
        '<div id="annotary-note-content" name="note"></div>' +
      '</div>';
    }
    else
      return; // Don't show the note if it's someone else's highlights and there's no note

    $('body').append('<div id="annotary-note">' +
      '<div id="annotary-note-menubar" class="annotary-menubar">' +
        '<a id="annotary-note-close" class="annotary-menubar-close"></a>' +
      '</div>' +
      html +
    '</div>');

    // Position it at the y-offset of the highlight and the x-offset of the right side of the common ancestor
    var highlightFirst = $('.annotary-highlight-id-' + id).first();
    var ancestor = $('.annotary-highlight-id-' + id).commonAncestor();
    var top = highlightFirst.offset().top - 20;
    if (top < 20)
      top = 20;
    var left = ancestor.offset().left + ancestor.width();
    if (left > $(window).width() - 220)
      left = $(window).width() - 220;
    $('#annotary-note').css('top', top + 'px');
    $('#annotary-note').css('left', left + 'px');

    $('#annotary-note').draggable({
      handle: '#annotary-note-menubar'
    });

    // Populate the note or focus the textarea if it's empty
    if (window.bookmarkRegistry.currentHighlights[id].note == '')
      $('#annotary-note-content').focus();
    else
      $('#annotary-note-content').text(window.bookmarkRegistry.currentHighlights[id].note);

    $('#annotary-note-close')
      .css('background', 'url(' + window.browser.getURL('/shared/img/btn_close.png') + ')')
      .hover(function () {
        $(this).css('background', 'url(' + window.browser.getURL('/shared/img/btn_close_on.png') + ')');
      }, function () {
        $(this).css('background', 'url(' + window.browser.getURL('/shared/img/btn_close.png') + ')');
      })
      .click(function (obj) {
        return function () {
          obj.closeNoteWindow();
        }
      }(this));

    $('#annotary-note-content').keyup(function (obj) {
      return function () {
        obj.noteChanged();
      }
    }(this));

    // Stops the click event from getting to the document and closing the note window
    $('#annotary-note').mousedown(function(event){
      event.stopPropagation();
    });

    this.noteOpen = id;

    // Display the remove icon
    this.displayRemoveIcon(id);


  },

  removeHighlight: function (highlightId) {

    this.closeNoteWindow();

    highlight = window.bookmarkRegistry.currentHighlights[highlightId];
    this.clearHighlight(highlight);

    window.api.request({type: 'DELETE', uri: '/highlights/'+highlightId+'.json'});

  },

  clearHighlight: function (highlight) {

    var highlightApplier = rangy.createCssClassApplier('annotary-highlight-id-' + highlight.id, true);
    $('.annotary-highlight-note-' + highlight.id).remove();
    $('.annotary-highlight-remove-' + highlight.id).remove();
    $('.annotary-highlight-id-' + highlight.id).removeClass('annotary-highlight');
    $('.annotary-highlight-id-' + highlight.id).removeClass('annotary-highlight-clickable');
    highlightApplier.undoToRange(highlight.range);
    delete window.bookmarkRegistry.currentHighlights[highlight.id];

  },

  closeNoteWindow: function () {

    clearTimeout(this.noteSaveTimeout);
    this.saveNote();
    $('#annotary-note').remove();
    var noteId = this.noteOpen;
    this.noteOpen = null;
    if (noteId != this.hoveredHighlight)
      this.hideRemoveIcon(noteId);

    // Hacky fix for issue where focus goes to the bottom of the page after the note closes in Firefox, so arrow keys don't work to scroll
    $('.annotary-highlight-id-' + noteId + ':last').attr('tabindex','999999').focus().blur().removeAttr('tabindex');

  },

  noteChanged: function () {

    clearTimeout(this.noteSaveTimeout);
    this.noteSaveTimeout = setTimeout(function (obj) {
      return function () {
        obj.saveNote();
      }
    }(this), 1000);

  },

  setNoteIcon: function (id) {

    if (window.bookmarkRegistry.currentHighlights[id].note == '')
      $('.annotary-highlight-note-' + id).remove();
    else if ($('.annotary-highlight-note-' + id).length == 0) {
      $('.annotary-highlight-id-' + id)
        .last()
        .after(
          $('<img class="annotary-highlight-note" />')
            .addClass('annotary-highlight-note-' + id)
            .attr('src', window.browser.getURL('/shared/img/icon_comment.png')
          )
        );
      $('.annotary-highlight-note-' + id).click(function (obj, id) {
        return function (event) {
          obj.openNoteWindow(id);
          event.preventDefault();
          event.stopPropagation();
        }
      }(this, id));
    }

  },

  setRemoveIcon: function (id) {

    if (window.bookmarkRegistry.currentBookmark != window.bookmarkRegistry.myBookmark)
      return;

    if ($('.annotary-highlight-remove-' + id).length == 0) {

      if (window.bookmarkRegistry.currentHighlights[id].note == '')
        var sel = $('.annotary-highlight-id-' + id).last();
      else
        var sel = $('.annotary-highlight-note-' + id);

      sel.after(
        $('<span class="annotary-highlight-remove"></span>')
          .addClass('annotary-highlight-remove-' + id)
          .append($('<img />').attr('src', window.browser.getURL('/shared/img/icon_trash.png'))
        )
      );

    }

    $('.annotary-highlight-id-' + id + ', .annotary-highlight-note-' + id + ', .annotary-highlight-remove-' + id).live({mouseenter: function (obj, id) {
      return function () {
        obj.hoveredHighlight = id;
        obj.displayRemoveIcon(id);
      }
    }(this, id), mouseleave: function (obj, id) {
      return function () {
        obj.hoveredHighlight = null;
        setTimeout(function (obj, id) {
          return function () {
            if (obj.hoveredHighlight != id)
              obj.hideRemoveIcon(id);
          }
        }(obj, id), 500);
      }
    }(this, id)});

    $('.annotary-highlight-remove-' + id).unbind('click').click(function (obj, id) {
      return function (event) {
        obj.removeHighlight(id);
        event.stopPropagation();
        event.preventDefault();
      }
    }(this, id));

  },

  displayRemoveIcon: function (id) {

    if (window.bookmarkRegistry.currentBookmark == window.bookmarkRegistry.myBookmark)
      $('.annotary-highlight-remove-' + id).show();

  },

  hideRemoveIcon: function (id) {

    if (this.noteOpen != id)
      $('.annotary-highlight-remove-' + id).hide();

  },

  saveNote: function () {

    if (this.noteOpen == null)
      return;

    var note = $('#annotary-note-content').val();

    if (note == window.bookmarkRegistry.currentHighlights[this.noteOpen].note)
      return;

    window.bookmarkRegistry.currentHighlights[this.noteOpen].note = note;
    this.setNoteIcon(this.noteOpen);

    window.api.request({type: 'PUT', uri: '/highlights/' + this.noteOpen + '.json', data: {note: note}});

  },

  saveRange: function (range) {

    var start = this.savePoint(range.startContainer, range.startOffset);
    var end = this.savePoint(range.endContainer, range.endOffset);
    var fragment = range.cloneContents();
    start.highlightText = fragment.childNodes[0].textContent; // Highlighted portion of the start node (text after the offset point)
    end.highlightText = fragment.childNodes[fragment.childNodes.length-1].textContent; // Highlighted portion of the end node (text before the offset point)
    return {
      startPoint: start,
      endPoint: end,
      text: range.toString()
    };

  },

  savePoint: function (container, offset) {

    var point = {}
    var el = container.parentElement;
    point.elSelector = this.getElementSelector(el); // Generic non-unique jquery selector for parent element
    point.elText = $(el).text(); // Overall text content of the parent element

    // Get the "real" nodes, which ignore the markup introduced by the highlight class applier
    // Then find the real node that includes the point
    var realNodes = this.getRealNodes(el);
    $.each(realNodes, function (i, realNode) {

      var aggregateOffset = 0;
      $.each(realNode.nodes, function (j, node) {
        if (container == node) {
          point.nodeIndex = i; // Index of the real node that includes the point
          point.nodeText = realNode.text; // Text of the node that containst the point
          point.offset = offset + aggregateOffset; // Number of characters into the node that the point occurs
          return false;
        }
        aggregateOffset += $(node).text().length;
      });

      if (point.offset != undefined)
        return false;

    });

    return point;

  },

  restoreRange: function (savedRange) {

    startPoint = this.restorePoint(savedRange.startPoint, false);
    endPoint = this.restorePoint(savedRange.endPoint, true);

    if (startPoint.length == 0 || endPoint.length == 0)
      return console.log('Range not found: ' + savedRange.text);

    // If we have multiple start or end points, try out all possible ranges and get the one that closest matches the saved text
    ranges = [];
    for (i in startPoint) {
      for (j in endPoint) {

        var range = {
          startPoint: startPoint[i],
          endPoint: endPoint[j],
          range: rangy.createRange()
        };
        try {
          range.range.setStart(range.startPoint.node, range.startPoint.offset);
          range.range.setEnd(range.endPoint.node, range.endPoint.offset);
          range.text = range.range.toString();
        }
        catch (e) {
          return console.log('Error restoring range (' + e.message + '): ' + savedRange.text);
        }

        // Ignore ranges that aren't even close to the right length so we don't get bogged down with levenshtein
        var lengthdiff = range.text.length - savedRange.text.length;
        if (lengthdiff > 10 || lengthdiff < -10)
          continue;

        var parentEl = $(range.range.commonAncestorContainer.parentElement);
        range.visible = parentEl.is(':visible') && parentEl.offset().top + parentEl.height() > 0 && parentEl.offset().left + parentEl.width() > 0;

        range.lev = this.levenshtein(range.text, savedRange.text);
        ranges.push(range);

      }
    }

    var range = null;
    for (i in ranges) {
      if (range == null || (ranges[i].visible && !range.visible) || ranges[i].lev < range.lev)
        range = ranges[i];
    }

    // If the closest matching range is closer than 10 in levenshtein distance, return the range
    if (range.lev < 10)
      return range.range;
    else
      console.log('Range not found: ' + savedRange.text);
      // TODO: If there's no match, do a general search for the string in the document and highlight it instead
      // TODO: Do something with the ranges that weren't found, rather than just logging the miss

  },

  restorePoint: function (savedPoint, isEndPoint) {

    elements = this.restorePointElements(savedPoint);
    if (elements.length == 0)
      return [];

    var points = [];
    var obj = this;

    // Loop through elements that matched and add possible matching points to the points array
    $.each(elements, function (i, el) {

      var realNodes = obj.getRealNodes(el);
      var matchingRealNodes = [];

      // Check if the node at the specified index matches the saved text
      if (realNodes.length > savedPoint.nodeIndex && realNodes[savedPoint.nodeIndex].text == savedPoint.nodeText)
        matchingRealNodes = [realNodes[savedPoint.nodeIndex]];

      // If we didn't find the right real node at the index, look for an exact text match
      if (matchingRealNodes.length == 0) {

        $.each(realNodes, function (i, realNode) {
          if (realNode.text == savedPoint.nodeText)
            matchingRealNodes.push(realNode);
        });

      }

      // If there's no exact text match, look for real nodes that contain the highlighted portion of the text
      if (matchingRealNodes.length == 0) {

        $.each(realNodes, function (i, realNode) {
          var index = realNode.text.indexOf(savedPoint.highlightText);
          if (index > -1) {
            // If realnode containing endpoint is an element, the offset is the number of nodes in the element (since the point will only be an element if the range contained the entire contents)
            if (isEndPoint && realNode.nodes.length == 1 && realNode.nodes[0].nodeType == 1)
              realNode.offset = realNode.nodes[0].childNodes.length;
            // If realnode is a text node, the startpoint offset is the index of the highlight, and the endpoint offset is the index plus the length of the highlighted portion of the text
            else
              realNode.offset = isEndPoint ? index + savedPoint.highlightText.length : index;
            matchingRealNodes.push(realNode);
          }
        });

      }

      // For each matching real node, identify the actual DOM node that contains the offset, then return a list of all matching points
      $.each(matchingRealNodes, function (i, realNode) {

        // If the offset is specified in the real node, use that offset - otherwise, use the default
        var offset = typeof realNode.offset == 'undefined' ? savedPoint.offset : realNode.offset;

        $.each(realNode.nodes, function (i, node) {

          var nodeText = $(node).text();
          if (nodeText.length >= offset) {
            if (node.nodeType == 3 || node.nodeType == 1)
              points.push({node: node, offset: offset})
            return false;
          }
          else
            offset -= nodeText.length;

        });

      });

    });

    return points;

  },

  restorePointElements: function (savedPoint) {

    // Get all elements that match the jquery selector plus any elements that contain the highlighted portion of the nodeText
    var elements = [];
    $('*')
      .filter(function (highlightText) {
        return function () {
          // Check that the element contains the text
          if ($(this).text().indexOf(highlightText) < 0)
            return false;
          // Check that its children do not
          var childrenLength = $(this).children().filter(function (highlightText) {
            return function () {
              return $(this).text().indexOf(highlightText) > -1 ? true : false;
            }
          }(highlightText)).length;
          return childrenLength == 0 ? true : false;
        }
      }(savedPoint.highlightText))
      .add(savedPoint.elSelector)
      .each(function () {
        elements.push({el: this, text: $(this).text()});
      });

    // Remove elements that are clearly not correct, and identify if any are a perfect match
    var possibleMatches = [];
    var perfectMatches = [];
    for (i in elements) {

      var elText = elements[i].text;
      var elTextLength = typeof elText == 'undefined' ? 0 : elText.length;
      var savedText = savedPoint.elText;
      var lengthRatio = elTextLength / savedText.length;

      // If they're identical, we're all set
      if (elText == savedText) {
        perfectMatches.push(elements[i].el);
        continue;
      }

      // If the element has no text, it's not the right one
      if (!elText)
        continue;

      // If one of the comparison strings is more than twice as long, assume it's not the right one
      else if (lengthRatio > 2 || lengthRatio < 0.5)
        continue;

      possibleMatches.push(elements[i]);

    }

    if (perfectMatches.length > 0)
      return perfectMatches;

    if (possibleMatches.length == 0)
      return [];

    if (possibleMatches.length == 1)
      return [possibleMatches[0].el];

    // If we haven't found a perfect match, look for elements that contain the highlight text
    probableMatches = [];
    for (i in possibleMatches) {
      if (possibleMatches[i].text.indexOf(savedPoint.highlightText) > -1)
        probableMatches.push(possibleMatches[i].el);
    }

    if (probableMatches.length > 0)
      return probableMatches;
    else
      return [];

  },

  // Get an array of the "real" nodes in an element, independent of markup introduced by highlighting
  // Each node is a dictionary containing the text, and an array of the DOM nodes that compose the real node (including the highlighting nodes)
  getRealNodes: function (el) {

    var realNodes = [];
    var currentRealNode = {nodes: [], text: ''};
    $(el).contents().each(function (i, node) {

      // If it's a text node or part of a highlight, tack it onto the currentRealNode
      if (node.nodeType == 3 || (node.nodeType == 1 && $(node).is('.annotary-highlight, .annotary-highlight-note, .annotary-highlight-remove'))) {
        currentRealNode['nodes'].push(node);
        currentRealNode['text'] += $(node).text();
      }

      // Otherwise, it's a real node, so save the currentRealNode and push this one on there too
      else {
        if (currentRealNode.nodes.length > 0) {
          realNodes.push(currentRealNode);
          currentRealNode = {nodes: [], text: ''};
        }
        realNodes.push({nodes: [node], text: $(node).text()})
      }

    });

    // Tack on the currentRealNode if it wasn't saved already
    if (currentRealNode.nodes.length > 0)
      realNodes.push(currentRealNode);

    return realNodes;

  },

  // Get a fairly generic but unlikely to change jquery selector for an element (even if the page content changes pretty considerably)
  // Doesn't need to be unique, just needs to narrow down the field a little bit
  // More important that it's pretty robust even if the page changes a lot
  getElementSelector: function (el) {

    // TODO: Would be useful to check how many elements match the selector we've made up and try to come up with one that
    // narrows it down to only 10 or fewer elements, like if we end up with a selector TR > TD there's going to be a ton
    // of matches, so maybe go up to TABLE.class > TR > TD in that case...

    var tag = el.tagName;
    var id = el.id == '' ? null : el.id;

    if (id)
      return tag + '#' + id;

    var ptag = el.parentElement.tagName;
    var pid = el.parentElement.id == '' ? null : el.parentElement.id;

    if (pid)
      return ptag + '#' + pid + ' > ' + tag;

    var classes = [];
    var classSplit = el.className.split(/\s+/);
    for (i in classSplit) {
      if (classSplit[i] != '')
        classes.push(classSplit[i]);
    }

    if (classes.length > 0)
      return tag + '.' + classes.join('.');

    var pclasses = [];
    var classSplit = el.parentElement.className.split(/\s+/);
    for (i in classSplit) {
      if (classSplit[i] != '')
        pclasses.push(classSplit[i]);
    }

    if (pclasses.length > 0)
      return ptag + '.' + pclasses.join('.') + ' > ' + tag;

    return ptag + ' > ' + tag;

  },

  levenshtein: function (s1, s2) {

    // http://kevin.vanzonneveld.net
    // + original by: Carlos R. L. Rodrigues (http://www.jsfromhell.com)
    // + bugfixed by: Onno Marsman
    // + revised by: Andrea Giammarchi (http://webreflection.blogspot.com)
    // + reimplemented by: Brett Zamir (http://brett-zamir.me)
    // + reimplemented by: Alexander M Beedie
    // * example 1: levenshtein('Kevin van Zonneveld', 'Kevin van Sommeveld');
    // * returns 1: 3

    if (s1 == s2) {
      return 0;
    }

    var s1_len = s1.length;
    var s2_len = s2.length;
    if (s1_len === 0) {
      return s2_len;
    }
    if (s2_len === 0) {
      return s1_len;
    }

    // BEGIN STATIC
    var split = false;
    try {
      split = !('0')[0];
    } catch (e) {
      split = true; // Earlier IE may not support access by string index
    }
    // END STATIC
    if (split) {
      s1 = s1.split('');
      s2 = s2.split('');
    }

    var v0 = new Array(s1_len + 1);
    var v1 = new Array(s1_len + 1);

    var s1_idx = 0,
      s2_idx = 0,
      cost = 0;
    for (s1_idx = 0; s1_idx < s1_len + 1; s1_idx++) {
      v0[s1_idx] = s1_idx;
    }
    var char_s1 = '',
      char_s2 = '';
    for (s2_idx = 1; s2_idx <= s2_len; s2_idx++) {
      v1[0] = s2_idx;
      char_s2 = s2[s2_idx - 1];

      for (s1_idx = 0; s1_idx < s1_len; s1_idx++) {
        char_s1 = s1[s1_idx];
        cost = (char_s1 == char_s2) ? 0 : 1;
        var m_min = v0[s1_idx + 1] + 1;
        var b = v1[s1_idx] + 1;
        var c = v0[s1_idx] + cost;
        if (b < m_min) {
          m_min = b;
        }
        if (c < m_min) {
          m_min = c;
        }
        v1[s1_idx + 1] = m_min;
      }
      var v_tmp = v0;
      v0 = v1;
      v1 = v_tmp;
    }
    return v0[s1_len];

  },

}
