// Firefox Libs
const ff_self = require("self");
const ff_tabs = require("tabs");
const ff_toolbarbutton = require("toolbarbutton");
const ff_xhr = require("xhr");
var {env} = require('api-utils/environment');
var ff_is_dev = env.IS_DEV == 1 ? true : false;
var {Cc, Ci} = require("chrome");
var ff_cookieSvc = Cc["@mozilla.org/cookieService;1"].getService(Ci.nsICookieService);
var ff_ioService = Cc["@mozilla.org/network/io-service;1"].getService(Ci.nsIIOService);
var ff_btn = null; // Instantiated by the browserapi setBrowserButtonListener
var ff_tab_registry = {};
var ff_worker_registry = {};
var ff_next_tab_id = 0;
var ff_message_listener = null; // Callback function for messages sent to this environment - set by the browserapi from the shared libs

exports.main = function(options) {

  // Global scope
  options = options;

  // Create a fake window object for my globals
  window = {};

  var loadContentScripts = function (tab) {

    // Save the tab data for easy access later on
    var tab_id = ff_next_tab_id++;
    ff_tab_registry[tab_id] = tab;

    // If it's the login tab and login is complete, close it before loading scripts to prevent error message
    if (window.api !== undefined && tab_id != 0 && tab_id == window.api.loginTabId && tab.url.indexOf('login') == -1) {
      window.api.restoreSession();
      return;
    }

    // Add the content scripts and save a worker for sending and receiving messages
    var worker = ff_worker_registry[tab_id] = tab.attach({
      contentScriptFile: [
        ff_self.data.url("shared/js/jquery.js"),
        ff_self.data.url("shared/js/jquery-ui.js"),
        ff_self.data.url("shared/js/jquery.commonancestor.js"),
        ff_self.data.url("shared/js/rangy/rangy-core.js"),
        ff_self.data.url("shared/js/rangy/rangy-cssclassapplier.js"),
        ff_self.data.url("shared/js/rangy/rangy-selectionsaverestore.js"),
        ff_self.data.url("shared/js/rangy/rangy-serializer.js"),
        ff_self.data.url("shared/js/browserapi.js"),
        ff_self.data.url("shared/js/highlighter.js"),
        ff_self.data.url("shared/js/collections-picker.js"),
        ff_self.data.url("shared/js/content-apiclient.js"),
        ff_self.data.url("shared/js/content-messenger.js"),
        ff_self.data.url("shared/js/bookmark-registry.js"),
        ff_self.data.url("shared/js/content-toolbar.js"),
        ff_self.data.url("shared/js/content-bootstrap.js")
      ],
      contentScript: "ff_is_dev=" + (ff_is_dev ? 'true' : 'false') + ";$('head').append('<link rel=\"stylesheet\" type=\"text/css\" href=\"" + ff_self.data.url("shared/extension.css") + "\" />');",
    });

    // Create the background listener that corresponds to this content script, emulating chrome's system
    worker.port.on("default", function (tab_id, worker) {
      return function (request) {
        ff_message_listener(request, {tab:{id: tab_id}}, function (worker, request) {
          return function (response) {
            worker.port.emit("reply-" + request.id, response);
          }
        }(worker, request));
      }
    }(tab_id, worker));

  }

  // Since we're running this now for the first time, add the content scripts to all tabs that exist (for installation)
  for (i in ff_tabs) {
    var tab = ff_tabs[i];
    if (!tab.url.match(/^about:.*$/i))
      loadContentScripts(tab);
  }

  ff_tabs.on('ready', function(tab) {
    // Delete old registered tab and worker
    var tabId = window.browser.getTabId(tab);
    delete ff_tab_registry[tabId];
    delete ff_worker_registry[tabId];
    // If this is the login tab, increment the login tab id
    if (tabId != 0 && tabId != null && tabId == window.api.loginTabId)
      window.api.loginTabId = ff_next_tab_id;
    // Load new content scripts if the tab has a page in it
    if (!tab.url.match(/^about:.*$/i))
      loadContentScripts(tab);
  });

  ff_tabs.on('close', function (tab) {
    // Delete registered tab and worker
    var tabId = window.browser.getTabId(tab);
    delete ff_tab_registry[tabId];
    delete ff_worker_registry[tabId];
  });


// Include all the background scripts (use python script compile.py to generate this part)
//---PYTHON-INCLUDE-BACKGROUND-SCRIPTS---//
BrowserApi = function () {

  if (typeof chrome != 'undefined' && typeof chrome.extension != 'undefined')
    this.browser = 'chrome';
  else if (typeof ff_self != 'undefined' || typeof self.port != 'undefined')
    this.browser = 'firefox';
  else if (typeof safari != 'undefined' && typeof safari.extension != 'undefined')
    this.browser = 'safari';
  else
    this.browser = null; // For when it's not running in an extension

}

BrowserApi.prototype = {

  browser: null,
  message_reply_callbacks: {}, // Used to register reply callbacks when sending messages that get replies
  default_message_callback: function () {}, // Used to store the callback for receiving default messages in safari

  getHost: function () {

    switch (this.browser) {

      case 'chrome':

        var extensionId = chrome.i18n.getMessage('@@extension_id');
        if (extensionId == 'fncckipieakpgpjifiadfiigdeanbond') // Chrome store version
          return 'https://annotary.com';
        else
          return 'https://localdev.annotary.com';
        break;

      case 'firefox':

        if (ff_is_dev)
          return 'https://localdev.annotary.com';
        else
          return 'https://annotary.com';
        break;

      case 'safari':

        // TODO: Dev vs production for safari extension
        return 'https://annotary.com';
        break;

      default:

        return 'https://' + document.domain;
        break;

    }

  },

  getURL: function (url) {

    switch (this.browser) {

      case 'chrome':
        return chrome.extension.getURL(url);
        break;

      case 'firefox':
        return "resource://jid1-d0wnmlonblgjig-at-jetpack/annotary/data" + url;
        break;

      case 'safari':
        return safari.extension.baseURI + url.substring(1);
        break;

      default:
        return SHARED_LIBS_URL + url;

    }

  },

  getCookie: function (cookie, callback) {

    switch (this.browser) {

      case 'chrome':
        chrome.cookies.get(cookie, callback);
        break;

      case 'firefox':
        var uri = ff_ioService.newURI(cookie.url, null, null);
        var name = cookie.name;
        var cookieStr = ff_cookieSvc.getCookieStringFromHttp(uri, null, null); // TODO: This function could be removed in the future, and may need to be replaced by using getCookieFromString with an instance of Ci.nsIHttpChannelInternal as the channel. See https://developer.mozilla.org/en/XPCOM_Interface_Reference/nsICookieService#section_8
        if (!cookieStr)
          cookieStr = '';
        var cookieAr = cookieStr.split(';');
        for (var cookie in cookieAr) {
          cookie = cookieAr[cookie];
          cookie = cookie.replace(/^\s\s*/, '').replace(/\s\s*$/, ''); //Trim
          cookie = cookie.split('=');
          if (cookie[0] == name) {
            if (cookie[1] == '')
              return callback(null);
            else
              return callback(cookie[1]);
          }
        }
        return callback(null);
        break;

      // TODO: A way to check and delete cookies in safari, or another workaround to block API calls when logged out?

      default:
        callback('Fake Cookie nom nom nom');
        break;

    }

  },

  removeCookie: function (cookie) {

    switch (this.browser) {

      case 'chrome':
        chrome.cookies.remove(cookie);
        break;

      case 'firefox':
        var uri = ff_ioService.newURI(cookie.url, null, null);
        var name = cookie.name;
        ff_cookieSvc.setCookieStringFromHttp(uri, null, null, name + '=', '01/01/12 00:00', null); // TODO: This function could be removed in the future, and may need to be replaced by using setCookieString with an instance of Ci.nsIHttpChannelInternal as the channel. See https://developer.mozilla.org/en/XPCOM_Interface_Reference/nsICookieService#section_13
        break;

      // TODO: A way to check and delete cookies in safari, or another workaround to block API calls when logged out?

    }

  },

  getAllTabs: function (callback) {

    switch (this.browser) {

      case 'chrome':
        chrome.windows.getAll({populate: true}, function (callback) {
          return function (windows) {
            var tabs = [];
            for (i in windows)
              tabs = tabs.concat(windows[i].tabs);
            callback(tabs);
          }
        }(callback));
        break;

      case 'firefox':
        var tabs = [];
        for (i in ff_tabs) {
          tabs.push(ff_tabs[i]);
        }
        callback(tabs);
        break;

      case 'safari':
        var tabs = [];
        for (i in safari.application.browserWindows) {
          tabs = tabs.concat(safari.application.browserWindows[i].tabs);
        }
        callback(tabs);
        break;

    }

  },

  createOrOpenLoginTab: function () {

    var url = this.getHost() + '/login?utm_source=direct&utm_medium=extension';

    switch (this.browser) {

      case 'chrome':
        var urlpattern = '*://*.annotary.com/login*';
        chrome.tabs.query({url: urlpattern}, function (url) {
          return function (tabs) {
            if (tabs.length == 0) {
              var top = screen.height/2 - 300;
              var left = screen.width/2 - 300;
              chrome.windows.create({
                url: url,
                top: top,
                left: left,
                width: 600,
                height: 600,
                focused: true,
                type: 'popup'
              }, function (win) {
                window.api.loginTabId = win.tabs[0].id;
              })
            }
            else {
              for (j in tabs)
                chrome.tabs.update(tabs[j].id, {selected: true}); // Select if it's already open
            }
          }
        }(url));
        break;

      case 'firefox':
        var urlpattern = /^https?:\/\/(|[^.\/]+\.)annotary\.com\/login/i;
        for (i in ff_tabs) {
          if (ff_tabs[i].url.match(urlpattern))
            return ff_tabs[i].activate();
        }
        ff_tabs.open({
          url: url,
          onOpen: function (obj) {
            return function (tab) {
              window.api.loginTabId = ff_next_tab_id;
            }
          }(this)
        });
        break;

      case 'safari':
        var urlpattern = /^https?:\/\/(|[^.\/]+\.)annotary\.com\/login/i;
        for (i in safari.application.browserWindows) {
          var browserWindow = safari.application.browserWindows[i];
          for (j in browserWindow.tabs) {
            var tab = browserWindow.tabs[j];
            if (tab.url && tab.url.match(urlpattern)) {
              browserWindow.activate();
              tab.activate();
              return;
            }
          }
        }
        window.api.loginTabId = safari_next_tab_id;
        newTab = safari.application.activeBrowserWindow.openTab();
        newTab.url = url;
        return newTab.activate();
        break;

    }

  },

  updateTab: function (tab, update) {

    switch(this.browser) {

      case 'chrome':
        chrome.tabs.update(tab.id, update);
        break;

      case 'firefox':
        if (typeof update.url != 'undefined')
          tab.url = update.url;
        break;

      case 'safari':
        if (typeof update.url != 'undefined')
          tab.url = update.url;
        break;

    }

  },

  closeTab: function (tabId) {

    switch(this.browser) {

      case 'chrome':
        chrome.tabs.remove(tabId);
        break;

      case 'firefox':
        ff_tab_registry[tabId].close();
        break;

      case 'safari':
        safari_tab_registry[tabId].close();
        break;

    }

  },

  isNewTab: function (tab) {

    switch (this.browser) {

      case 'chrome':
        return tab.url == 'chrome://newtab/' ? true : false;
        break;

      case 'firefox':
        return tab.url.match(/^about:.*$/) ? true : false;
        break;

      case 'safari':
        return (typeof tab.url == 'undefined') ? true : false;
        break;

    }

  },

  setBrowserButtonListener: function (callback) {

    switch (this.browser) {

      case 'chrome':
        chrome.browserAction.onClicked.addListener(callback);
        break;

      case 'firefox':
        // No button by default in Firefox, so create one in the global scope
        ff_btn = ff_toolbarbutton.ToolbarButton({
          id: 'annotary-toolbar-button',
          label: 'Annotary Toolbar',
          image: ff_self.data.url('icon.png'),
          onCommand: function (callback) {
            return function () {
              return callback(ff_tabs.activeTab);
            }
          }(callback)
        });
        ff_btn.moveTo({
          toolbarID: "nav-bar",
          forceMove: options.loadReason == 'install'
        });
        break;

      case 'safari':
        safari.application.addEventListener("command", function (e) {
          if (e.command == 'toolbarButton')
            callback(e.target.browserWindow.activeTab);
        }, false);
        break;

    }

  },

  setMessageListener: function (callback) {

    switch (this.browser) {

      case 'chrome':
        chrome.extension.onRequest.addListener(callback);
        break;

      case 'firefox':
        if (typeof ff_message_listener != 'undefined') // Background script just sets the callback as a variable, since it must create a new listener for each tab
          ff_message_listener = callback;
        else { // Content script
          self.port.on("default", function (callback) {
            return function (request) {
              callback(request, null, function (request) {
                return function (response) {
                  self.port.emit("reply-" + request.id, response);
                }
              }(request));
            }
          }(callback));
        }
        break;

      case 'safari':

        if (typeof safari.application != 'undefined') { // Background script
          this.default_message_callback = callback;
          safari.application.addEventListener("message", function (obj) {
            return function (e) {
              if (e.name == 'default') {
                var request = e.message;
                var sender = {tab: {id: obj.getTabId(e.target)}};
                obj.default_message_callback(request, sender, function (senderTab, request) {
                  return function (response) {
                    if (senderTab.page)
                      senderTab.page.dispatchMessage("reply-" + request.id, response);
                  }
                }(e.target, request));
              }
              else {
                var replyId = e.name.split('-')[1];
                var callback = obj.message_reply_callbacks[replyId];
                if (typeof callback == 'function')
                  callback(e.message);
              }
            }
          }(this), false);
        }
        else { // Content Script
          this.default_message_callback = callback;
          safari.self.addEventListener("message", function (obj) {
            return function (e) {
              if (e.name == 'default') {
                var request = e.message;
                obj.default_message_callback(request, null, function (request) {
                  return function (response) {
                    safari.self.tab.dispatchMessage("reply-" + request.id, response);
                  }
                }(request));
              }
              else {
                var replyId = e.name.split('-')[1];
                var callback = obj.message_reply_callbacks[replyId];
                if (typeof callback == 'function')
                  callback(e.message);
              }
            }
          }(this), false);
        }
        break;

    }

  },

  sendBackgroundRequest: function (request, callback) {

    switch (this.browser) {

      case 'chrome':
        chrome.extension.sendRequest(request, callback);
        break;

      case 'firefox':
        if (typeof callback == 'function')
          self.port.on("reply-" + request.id, callback);
        self.port.emit("default", request);
        break;

      case 'safari':
        if (typeof callback == 'function')
          this.message_reply_callbacks[request.id] = callback;
        safari.self.tab.dispatchMessage("default", request);
        break;

    }

  },

  sendTabRequest: function (tabId, request, callback) {

    switch (this.browser) {

      case 'chrome':
        chrome.tabs.sendRequest(tabId, request, callback);
        break;

      case 'firefox':
        var worker = ff_worker_registry[tabId];
        if (!worker)
          return;
        if (typeof callback == 'function')
          worker.port.on("reply-" + request.id, callback);
        worker.port.emit("default", request);
        break;

      case 'safari':
        var tab = safari_tab_registry[tabId];
        if (!tab)
          return;
        if (typeof callback == 'function')
          this.message_reply_callbacks[request.id] = callback;
        tab.page.dispatchMessage("default", request);
        break;

    }

  },

  setIcon: function (details) {

    switch (this.browser) {

      case 'chrome':
        chrome.browserAction.setIcon(details);
        break;

      case 'firefox':
        var url = ff_self.data.url(details.path.substring(1));
        ff_btn.setIcon({url: url});
        break;

      // Safari doesn't change the toolbar icon because it doesn't support color in the first place

    }

  },

  ajaxRequest: function (host, args, callback, errorCallback) {

    switch (this.browser) {

      case 'firefox':
        var xhr = new ff_xhr.XMLHttpRequest();
        xhr.open(args.type, host + '/api' + args.uri, true);
        xhr.onreadystatechange = function() {
          if (xhr.readyState != 4)
            return;
          if ((xhr.status + '').substring(0,1) != '2')
            return errorCallback(xhr);
          else if (xhr.status == 204)
            return callback(xhr.responseText);
          else
            return callback(JSON.parse(xhr.responseText));
        };
        if (args.data && args.data != {}) {
          var params = []
          for (k in args.data)
            params.push(encodeURIComponent(k) + '=' + encodeURIComponent(args.data[k]));
          xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
          xhr.send(params.join('&'));
        }
        else
          xhr.send();
        break;

      default:
        $.ajax({
          type: args.type,
          url: host + '/api' + args.uri,
          data: args.data,
          context: args.context,
          dataType: 'json',
          error: errorCallback,
          success: callback
        });
        break;

    }

  },

  getTabId: function (tab) {

    switch (this.browser) {

      case 'firefox':
        for (var tabId in ff_tab_registry) {
          if (tab === ff_tab_registry[tabId])
            return tabId;
        }
        return null;
        break;

      case 'safari':
        for (var tabId in safari_tab_registry) {
          if (tab === safari_tab_registry[tabId])
            return tabId;
        }
        return null;
        break;

      default:
        return tab.id;
        break;

    }

  }

}
var ApiClient = function () {

  this.host = window.browser.getHost();

}

ApiClient.prototype = {

  host: null,
  loginTabId: null,
  isAuthenticated: true,

  request: function (args, callback, errorCallback) {

    // args = {type:, uri:, data:, context:}

    // Check for the session cookie before making a request
    window.browser.getCookie({url: this.host, name: 'sessionid'}, function (obj, args, callback, errorCallback) {
      return function (cookie) {
        if (!cookie) {
          obj.authFailed();
          return;
        }
        window.browser.ajaxRequest(
          obj.host,
          args,
          function (obj, callback) {
            return function (response) {
              if (!obj.isAuthenticated) {
                obj.restoreSession();
              }
              callback(response);
            }
          }(obj, callback),
          function (obj, errorCallback) {
            return function (xhr) {
              if (xhr.status == 401) {
                obj.authFailed();
              }
              else if (xhr.status == 400) {
                var errors = JSON.parse(xhr.responseText);
                errorCallback(errors);
              }
              else if (xhr.status == 404) {
                errorCallback('Not Found');
              }
            }
          }(obj, errorCallback)
        );
      }
    }(this, args, callback, errorCallback));

  },

  // Series of steps to take when auth fails
  authFailed: function () {

    // Close the toolbar on all pages from the background script
    window.backgroundToolbar.close();

    // Delete the session cookie to block future requests
    window.browser.removeCookie({url: window.api.host, name: 'sessionid'});

    // Grey out the annotary icon
    this.isAuthenticated = false;
    window.browser.setIcon({path: '/icon_grey.png'});

  },

  // Opens up a tab with the login page so the user can log in again
  loginPage: function () {

    window.browser.createOrOpenLoginTab();

  },

  restoreSession: function () {

    this.isAuthenticated = true;
    // Close the login tab
    if (this.loginTabId) {
      window.browser.closeTab(this.loginTabId);
      this.loginTabId = null;
    }
    // Re-register bookmarks on all open pages
    window.messenger.sendRequest({type: 'func', inst: 'bookmarkRegistry', func: 'registerUrlBookmarks'});
    // Light up the icon again
    window.browser.setIcon({path: '/icon.png'});
    // Re-open the toolbar
    window.backgroundToolbar.open();

  }

}
LinkRegistry = function () {

}

LinkRegistry.prototype = {

  registry: {},

  register: function (url, vars) {

    this.registry[url] = vars;

  },

  clear: function (url) {

    if (this.registry[url])
      delete this.registry[url];

  }

}
function BackgroundToolbar() {

  // TODO: Get the most recent collection ID and delete placeholder below
  this.workingCollectionId = 3;

}

BackgroundToolbar.prototype = {

  isOpen: false,
  workingCollectionId: null,

  open: function (mode, tabId) {

    if (this.isOpen == true)
      return;

    window.messenger.sendRequest({type: 'func', inst: 'contentToolbar', func: 'open'}, function (mode, tabId) {
      return function (responseTabId, response) {
        // In the specified tab, set the specified mode
        if (responseTabId == tabId && mode == 'bookmark')
          window.messenger.sendSingleTabRequest(tabId, {type: 'func', inst: 'contentToolbar', func: 'toggleBookmarkMode'});
        else if (responseTabId == tabId && mode == 'highlight')
          window.messenger.sendSingleTabRequest(tabId, {type: 'func', inst: 'contentToolbar', func: 'toggleHighlightMode'});
        else if (responseTabId == tabId && mode == 'network')
          window.messenger.sendSingleTabRequest(tabId, {type: 'func', inst: 'contentToolbar', func: 'toggleNetworkMode', args: ['onStateOnly']});
      }
    }(mode, tabId));

    this.isOpen = true;

  },

  close: function () {

    if (this.isOpen == false)
      return;

    window.messenger.sendRequest({type: 'func', inst: 'contentToolbar', func: 'close'});
    this.isOpen = false;

  },

  toggle: function (mode, tabId) {

    if (this.isOpen)
      this.close();
    else
      this.open(mode, tabId);

  },

  networkMode: function (tabId) {

    if (this.isOpen == false)
      return this.open('network', tabId);

    // Open network mode in just the selected tab
    window.messenger.sendSingleTabRequest(tabId, {type: 'func', inst: 'contentToolbar', func: 'toggleNetworkMode', args: ['onStateOnly']});

  },

  setWorkingCollectionId: function (id) {

    this.workingCollectionId = id;

  }

}
function BrowserButton () {

  window.browser.setBrowserButtonListener(function (obj) {
    return function(tab) {

      window.browser.getCookie({url: window.api.host, name: 'sessionid'}, function (obj, tab) {
        return function (cookie) {

          // If there's no active session, open up a tab with the login page
          if (!cookie || !window.api.isAuthenticated)
            return window.api.loginPage();

          // If this is the new tab page, make the new tab go to my profile on annotary
          if (window.browser.isNewTab(tab)) {
            window.browser.updateTab(tab, {url: 'https://annotary.com/profiles/me?utm_source=direct&utm_medium=extension'});
            return;
          }

          // Otherwise...

          // Save the selection in the current tab so it gets highlighted once the toolbar opens or once the bookmark is set
          var tabId = window.browser.getTabId(tab);
          window.messenger.sendSingleTabRequest(tabId, {type: 'func', inst: 'contentToolbar', func: 'saveSelection'}, function (tabId) {
            return function () {
              window.messenger.sendSingleTabRequest(tabId, {type: 'func', inst: 'bookmarkRegistry', func: 'showingSomeoneElsesBookmark'}, function (tabId) {
                return function (showingSomeoneElsesBookmark) {
                  // If we're displaying someone else's bookmark in the tab, open it without specifying a mode (the content toolbar opens all tabs showing highlights in network mode)
                  if (showingSomeoneElsesBookmark)
                    window.backgroundToolbar.toggle(null, tabId);
                  else {
                    window.messenger.sendSingleTabRequest(tabId, {type: 'func', inst: 'bookmarkRegistry', func: 'iHaveBookmarked'}, function (tabId) {
                      return function (isBookmarked) {
                        // If it's bookmarked, open up the toolbar with highlight mode in the active tab
                        if (isBookmarked)
                          window.backgroundToolbar.toggle('highlight', tabId);
                        // If not, open the toolbar with bookmark mode in the active tab
                        else
                          window.backgroundToolbar.toggle('bookmark', tabId);
                      }
                    }(tabId));
                  }
                }
              }(tabId));
            }
          }(tabId));

        }
      }(obj, tab));

    }
  }(this));

}

BrowserButton.prototype = {

}
BackgroundMessenger = function () {

  // Init listener for messages from content scripts
  window.browser.setMessageListener(function (obj) {
    return function(request, sender, sendResponse) {
      obj.handleRequest(request, sender, sendResponse);
    }
  }(this));

}

BackgroundMessenger.prototype = {

  nextRequestId: 0,

  // Handles a request from a content script
  handleRequest: function (request, sender, sendResponse) {

    if (request.type == 'func') {
      var args = request.args ? request.args : [];
      for (i in args) {
        if (args[i] == '__tab__')
          args[i] = sender.tab.id;
      }
      response = window[request.inst][request.func].apply(window[request.inst],args);
      return sendResponse(response);
    }

    else if (request.type == 'data') {
      response = window[request.inst][request.attr];
      return sendResponse(response);
    }

    else if (request.type == 'api') {
      var callback = function (response) {
        response = {
          status: 'success',
          data: response
        }
        sendResponse(response);
      }
      var errorCallback = function (response) {
        response = {
          status: 'error',
          data: response
        }
        sendResponse(response);
      }
      window.api.request(request.args, callback, errorCallback);
    }

  },

  // Sends a message to each tab's content script individually
  // If there's a callback, it runs for every tab, including th tab id as a parameter
  sendRequest: function (request, callback) {

    window.browser.getAllTabs(function (obj, callback) {
      return function (tabs) {
        for (i in tabs) {
          var tabId = window.browser.getTabId(tabs[i]);
          obj.sendSingleTabRequest(tabId, request, function (tabId, callback) {
            return function (response) {
              if (typeof callback == 'function')
                callback(tabId, response);
            }
          }(tabId, callback));
        }
      }
    }(this, callback));

  },

  // Sends a message to a single tab only
  sendSingleTabRequest: function (tabId, request, callback) {

    request.id = this.nextRequestId++;
    window.browser.sendTabRequest(tabId, request, callback);

  },

}
browser = window.browser = new BrowserApi();
messenger = window.messenger = new BackgroundMessenger();
linkRegistry = window.linkRegistry = new LinkRegistry();
backgroundToolbar = window.backgroundToolbar = new BackgroundToolbar();
browserButton = window.browserButton = new BrowserButton();
api = window.api = new ApiClient();

//---END-PYTHON-INCLUDE-BACKGROUND-SCRIPTS---//

};
